% FootprintReach_TEST_07262018
% Calculate footprint and reach metrics upon individual groups and at system level
%   FOOTPRINT   = fraction of each PRODUCER's production required to support a particular (target) CONSUMER group
%   REACH       = fraction of particular (target) PRODUCER group production going to support each CONSUMER group
% define TraceGroup
%       Footprint: TraceGroup = consumer
%       Reach:     TraceGroup = producer
% calls:
%       f_read_EwE_csv_04292016
%       f_AggregateResults_EwE_03122015
%       ECOTRANuncertainty_05062016
%       	f_ECOfunction_05142015              returns a single ECOTRAN model for 1 "type" EwE model or 1 MonteCarlo EwE model
%        	f_RedistributeCannibalism           remove cannibalism terms on diagonal of matrix EwE_diet
%        	f_CalcPredationMatrix               for each producer, p, and consumer, c: ((b_pc * Q_c) / M2_p)) = the fraction of total predation on each producer, p, going to each consumer, c
%     	f_E2E_pedigree_08032018             	generate the pedigree (CV) of each cell of the EnergyBudget_matrix (A_cp)
%     	f_E2E_MonteCarlo_08032018           	generate Monte Carlo models from the EnergyBudget_matrix (A_cp)
%       f_Footprint_07272018                    (FOOTPRINT)
%           f_ProductionTrace_07272018
%       f_Reach_01212018                        (REACH)
%           f_DietTrace_03152015
%           f_DietTraceDownward_03152015
%       p_WebPlotter_01032017
% returns:
%       mean_Footprint                          footprint of each consumer on each producer (array, consumers x producers)
%       std_Footprint
%       mean_TraceGroup_SystemFootprint         footprint of TraceGroup CONSUMER on system ([sum TraceGroup CONSUMER footprint on all PRODUCERS] / [total production of all PRODUCERS (no nutrients, no detritus, no import)]
%       std_TraceGroup_SystemFootprint
%       TraceGroup_SystemReach                  reach of TraceGroup PRODUCER on system (
%       mean_TraceGroup_SystemReach
%       std_TraceGroup_SystemReach
% revision date: 7-27-2018


% *************************************************************************
% STEP 1: load & aggregate EwE results-------------------------------------
% step 1a: define model to work with
ReadFile_directory    	= '/Users/ruzickaj/Documents/ECOTRAN_08052018/FoodWeb_models/TEST_model/';
readFile               	= 'TESTmodel_07252018.csv';
readFile               	= [ReadFile_directory readFile];

% step 1b: load EwE model from K-path (.csv format) -----------------------
dat                 	= f_read_EwE_csv_04292016(readFile); % use for models with 10 primary producers

% step 1c: aggregate model results & prep EwEResult for analysis ----------
[EwEResult, PEDIGREE] 	= f_AggregateResults_EwE_03122015(dat);





% *************************************************************************
% STEP 2: load pre-generated Monte Carlo EwE models------------------------
% step 2a: Identify Monte Carlo method ------------------------------------
%          method 1: start with the one original ECOTRAN base model and generate a set of Monte Carlo E2E models of the ECOTRAN production matrix using a predefined CV (or defualt pedigree CV = 0.5)
%          method 2: use pre-generated Monte Carlo EwE models, convert to ECOTRAN E2E models, and run scenarios directly on EACH of these individually
%                    (each matrix being its own base model and scenario model)

use_MonteCarlo          = 'n';  % SSS; use MonteCarlo uncertainty estimates?
MonteCarlo_method       = 1;    % SSS; choose 1 or 2

% step 2b: if method 2 is chosen, load MonteCarloSet ----------------------
if strcmp(use_MonteCarlo, 'y') && MonteCarlo_method == 2
    
    % define MonteCarloSet  = name of Monte Carlo results with file directory (loads 'MonteCarloStore')
    MonteCarloSet_FileDirectory = '';   % NOTE laptop MAC directory format
    MonteCarloSet               = '';   % sampling interval = +/1 1 STD sampling (1 std in denominator)
    MonteCarloSet               = [MonteCarloSet_FileDirectory MonteCarloSet];
    
    load(MonteCarloSet, 'MonteCarloStore')
else
    MonteCarloStore          = [];
end





% *************************************************************************
% STEP 3: ECOTRAN conversion-----------------------------------------------
% step 3a: perform ECOTRAN conversion -------------------------------------
[ECOTRAN]                                       = ECOTRANuncertainty_05062016(EwEResult, MonteCarloStore);

% step 3b: read in ECOTRAN structure variables ----------------------------
%          (so that no changes made to original values)
GroupType                                        = ECOTRAN.GroupType;
AggLabel                                         = ECOTRAN.AggLabel;
EnergyBudget_matrix                              = ECOTRAN.EnergyBudget_matrix;
EwE_Biomass                                      = ECOTRAN.EwE_Biomass;
EwE_PB                                           = ECOTRAN.EwE_PB;
EwE_QB                                           = ECOTRAN.EwE_QB;
DetritusFate_feces                               = ECOTRAN.DetritusFate_feces;
DetritusFate_senescence                          = ECOTRAN.DetritusFate_senescence;
ExcretionFate                                    = ECOTRAN.ExcretionFate;
EggFate                                          = ECOTRAN.EggFate;
ProductionLossScaler                             = ECOTRAN.ProductionLossScaler;
RetentionScaler                                  = ECOTRAN.RetentionScaler;     % sensitivity to advection & mixing (0 = more advection <--> less advection =1); (horizontal vector; without nutrient clms)
FunctionalResponseParams                         = ECOTRAN.FunctionalResponseParams;
[num_E2Egroups, E2E_clms, E2E_layers]            = size(EnergyBudget_matrix);
% TransferEfficiency                               = ECOTRAN.TransferEfficiency; % gets redefined manually below

% step 3c: find detritus, nutrients, BA & EM ------------------------------
%          by row address in EnergyBudget_matrix
looky_NUTRIENTS                                  = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYNitroNutr);
looky_NO3                                        = find(GroupType        == ECOTRAN.GroupTypeDef_NO3);
looky_plgcNH4                                    = find(GroupType        == ECOTRAN.GroupTypeDef_plgcNH4);
looky_bnthNH4                                    = find(GroupType        == ECOTRAN.GroupTypeDef_bnthNH4);
looky_PrimaryProducer                            = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYPrimaryProd);
looky_Macroalgae                                 = find(GroupType        == ECOTRAN.GroupTypeDef_Macrophytes);
looky_Phytoplankton                              = looky_PrimaryProducer(~ismember(looky_PrimaryProducer, looky_Macroalgae));
looky_DETRITUS                                   = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYDetritus);
looky_terminalPlgcDetr                           = find(GroupType        == ECOTRAN.GroupTypeDef_terminalPlgcDetr);
looky_terminalBnthDetr                           = find(GroupType        == ECOTRAN.GroupTypeDef_terminalBnthDetr);
looky_eggs                                       = find(GroupType        == ECOTRAN.GroupTypeDef_eggs);
looky_ANYConsumer                                = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYConsumer);
looky_fishery                                    = find(GroupType        == ECOTRAN.GroupTypeDef_fishery);
looky_nonNO3                                     = 1:E2E_clms;
looky_nonNO3(looky_NO3)                          = [];
% looky_nonNO3(looky_NUTRIENTS)                    = [];                        % SSS add NH4 if NH4 is also a physically imported driver
num_Nutrients                                    = length(looky_NUTRIENTS);
num_NH4                                          = length([looky_plgcNH4; looky_bnthNH4]);
num_PrimaryProducers                             = length(looky_PrimaryProducer);
num_Macroalgae                                   = length(looky_Macroalgae);
num_Phytoplankton                                = length(looky_Phytoplankton);
num_eggs                                         = length(looky_eggs);
num_detritus                                     = length(looky_DETRITUS);





% *************************************************************************
% STEP 4: Generate E2E Monte Carlo models based on ECOTRAN EnergyBudget_matrix--------------------------------------------
%         NOTE: this is Monte Carlo method 1
%         method 1: start with the one original ECOTRAN base model and generate a set of Monte Carlo E2E models of the ECOTRAN production matrix using a predefined CV (or defualt pedigree CV = 0.5)
num_webMonteCarlo                                           = 1000; % SSS set this value
if strcmp(use_MonteCarlo, 'y') && MonteCarlo_method == 1
    
    PEDIGREE.PEDIGREE_eggs                                  = 0.1; % SSS egg pedigree for all groups (CV)
    PEDIGREE.PEDIGREE_predation                             = 0.1; % SSS predation pedigree for all groups (CV)
    PEDIGREE.PEDIGREE_senescence                            = 0.1; % SSS senescence pedigree for all groups (CV)
    PEDIGREE.PEDIGREE_BacterialMTBLSM                       = 0.1; % SSS pedigree for implicit bacterial metabolism of terminal detritus (CV)
    PEDIGREE.PEDIGREE_NutrientUptake                        = 0.1; % SSS pedigree for nutrient uptake by primary producers (CV)
    ECOTRAN_PEDIGREE                                        = f_E2E_pedigree_08032018(EwEResult, PEDIGREE, ECOTRAN);
    
            % SSS use for standardized pedigree
            [rows, clms]                                = size(ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix);
            ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix     = 0.75 * ones(rows, clms);
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget       = zeros(7, clms);
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(1, :) = 0.25; % feces
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(2, :) = 0.25; % metabolism
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(3, :) = 0.25; % eggs
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(4, :) = 0.1; % predation
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(5, :) = 0.1; % senescence
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(6, :) = 0; % BA
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(7, :) = 0; % EM
    
    MonteCarloConditions.num_trials                         = num_webMonteCarlo;            % number of trials to run
    MonteCarloConditions.EnergyBudget_matrix                = ECOTRAN.EnergyBudget_matrix;
    MonteCarloConditions.DistributionType                   = 'normal';                     % 'normal' or 'uniform'
    [ConsumptionBudget_MonteCarlo, EnergyBudget_MonteCarlo] = f_E2E_MonteCarlo_08032018(MonteCarloConditions, ECOTRAN_PEDIGREE);

    EnergyBudget_matrix                                     = EnergyBudget_MonteCarlo;
    
    BioenergeticBudget_ProductionDetail(1, :, :)            = ConsumptionBudget_MonteCarlo(4, :, :);    % predation
    BioenergeticBudget_ProductionDetail(2, :, :)            = ConsumptionBudget_MonteCarlo(3, :, :);    % eggs
    BioenergeticBudget_ProductionDetail(3, :, :)            = ConsumptionBudget_MonteCarlo(5, :, :);    % senescence
    BioenergeticBudget_ProductionDetail(4, :, :)            = ConsumptionBudget_MonteCarlo(6, :, :);    % BA
    BioenergeticBudget_ProductionDetail(5, :, :)            = ConsumptionBudget_MonteCarlo(7, :, :);    % EM
    
    BioenergeticBudget(1, :, :)                             = sum(BioenergeticBudget_ProductionDetail); % production
    BioenergeticBudget(2, :, :)                             = ConsumptionBudget_MonteCarlo(1, :, :);    % feces
    BioenergeticBudget(3, :, :)                             = ConsumptionBudget_MonteCarlo(2, :, :);    % metabolism

    ECOTRAN.EnergyBudget_matrix                             = EnergyBudget_matrix;                      % plug the Monte Carlo EnergyBudget_matrix back into the ECOTRAN structure variable
    ECOTRAN.BioenergeticBudget_ProductionDetail             = BioenergeticBudget_ProductionDetail;     	% plug the Monte Carlo BioenergeticBudget_ProductionDetail back into the ECOTRAN structure variable
    ECOTRAN.BioenergeticBudget                              = BioenergeticBudget;                       % plug the Monte Carlo BioenergeticBudget back into the ECOTRAN structure variable
    
    E2E_layers                                              = MonteCarloConditions.num_trials;

end % (strcmp(use_MonteCarlo, 'y') && MonteCarlo_method == 1)





% *************************************************************************
% STEP 5: Production Required (FOOTPRINT)----------------------------------
%         NOTE: footprint calculations use ECOTRAN (E2E) model

% step 5a: matrix addresses of TESTmodel_07252018 groups (for easier plotting)
%          NOTE: these must be entered manually for each new model
NO3_footprint           = 1;
PlgcNH4_footprint       = 2;
BnthNH4_footprint       = 3;
SmlPhyto_footprint      = 4;
LargePhyto_footprint    = 5;
MicroZ_footprint        = 6;
MesoZ_footprint         = 7;
MacroZ_footprint        = 8;
HerbiJelly_footprint    = 9;
CarniJelly_footprint    = 10;
Euphausid_footprint     = 11;
Cephalo_footprint       = 12;
PlankFish_footprint     = 13;
PiscFish_footprint      = 14;
BenthFish_footprint     = 15;
JuvFish_footprint       = 16;
DepFBen_footprint       = 17;
CarnBen_footprint       = 18;
SuspFBen_footprint      = 19;
SeaBird_footprint       = 20;
GrayWhale_footprint     = 21;
BalWhal_footprint       = 22;
Pinniped_footprint      = 23;
Odontocete_footprint    = 24;
FishEggs_footprint      = 25;
InvertEggs_footprint    = 26;
Fishery1_footprint      = 27; % <<--note that fisheries come before terminal detritus
Fishery2_footprint      = 28; % <<--note that fisheries come before terminal detritus
PlgcDetr_footprint      = 29;
BnthcDetr_footprint     = 30;


% step 5b: % define TargetGroup as a consumer for Footprint analysis ------
TargetGroup_Footprint           = CarniJelly_footprint; % TargetGroup as a consumer for Footprint analysis

% step 5c: calculate footprint metrics ------------------------------------
% returns:
%       TraceGroup_FootprintVector  fraction of each producer group flowing to consumer = TargetGroup_Footprint; 
%                                       2 horizontal vectors: 
%                                           row 1 = mean across MonteCarlo models, 
%                                           row 2 = std across MonteCarlo models,
%                                           clms  = producer groups
%                                       (NOTE: use for web plotting footprint box colors)
%       TraceGroup_SystemFootprint  SYSTEM-LEVEL footprint = ratio of total TraceGroup footprint on all PRODUCERS to total production of all PRODUCER groups
%                                       2 horizontal vectors: 
%                                           row 1 = mean across MonteCarlo models,
%                                           row 2 = std across MonteCarlo models,
%                                           clms  = consumer groups
%       FootprintArray              fraction of each producer group flowing to each consumer 
%                                       (3D stack of matrices; rows = consumers X clms = producer X layers = MonteCarlo models)

TraceNonProduction              = 'n'; % account for non-production consumption? (include feces & metabolic costs of each trophic link?)

[TraceGroup_FootprintVector, TraceGroup_SystemFootprint, FootprintArray] ...
                                = f_Footprint_07272018(ECOTRAN, TargetGroup_Footprint, TraceNonProduction, use_MonteCarlo);

% step 5d: for web-plotting, need to put ECOTRAN-based FootprintVector into same order as EwEResult-based ReachVector (= TraceFraction_upward)
temp_FootprintVector                                        = TraceGroup_FootprintVector(1, :);         % fraction of each PRODUCER group's production flowing to TraceGroup = CONSUMER; use clm 1 = means of MonteCarlo models
looky_nutrients                                             = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYNitroNutr);
GroupType(looky_nutrients)                                  = [];
temp_FootprintVector(looky_nutrients)                       = [];
looky_fisheries                                             = find(floor(GroupType) == ECOTRAN.GroupTypeDef_fishery);
hold_FootprintFisheries                                     = temp_FootprintVector(looky_fisheries);    % pull out footprints on fisheries values
GroupType(looky_fisheries)                                  = [];                                       % delete fisheries
temp_FootprintVector(looky_fisheries)                       = [];                                       % delete fisheries
GroupType(end+1)                                            = ECOTRAN.GroupTypeDef_import;              % append import production group
temp_FootprintVector(end+1)                                 = 0;                                        % append import production group
GroupType((end+1):(end+length(looky_fisheries)))            = ECOTRAN.GroupTypeDef_fishery;             % append fisheries
temp_FootprintVector((end+1):(end+length(looky_fisheries))) = hold_FootprintFisheries;                  % append fisheries
FootprintVector                                             = temp_FootprintVector;





% *************************************************************************
% STEP 6: REACH (uses EwEresult)-------------------------------------------
% step 6a: matrix addresses of TESTmodel_07252018 groups (for easier plotting)
%          NOTE: these must be entered manually for each new model
%          NOTE: order differs from footprint because footprint uses ECOTRAN (E2E) model while reach uses ECOPATH (EwE) model
SmlPhyto_reach      = 1;
LargePhyto_reach    = 2;
MicroZ_reach        = 3;
MesoZ_reach         = 4;
MacroZ_reach        = 5;
HerbiJelly_reach    = 6;
CarniJelly_reach    = 7;
Euphausid_reach     = 8;
Cephalo_reach       = 9;
PlankFish_reach     = 10;
PiscFish_reach      = 11;
BenthFish_reach     = 12;
JuvFish_reach       = 13;
DepFBen_reach       = 14;
CarnBen_reach       = 15;
SuspFBen_reach      = 16;
SeaBird_reach       = 17;
GrayWhale_reach     = 18;
BalWhal_reach       = 19;
Pinniped_reach      = 20;
Odontocete_reach    = 21;
FishEggs_reach      = 22;
InvertEggs_reach    = 23;
PlgcDetr_reach      = 24;
BnthcDetr_reach     = 25;
import_reach        = 26; % <<--NOTE: import prey comes AFTER terminal detritus & BEFORE fisheries
Fishery1_reach      = 27; % <<--NOTE: fisheries come AFTER terminal detritus & import
Fishery2_reach      = 28; % <<--NOTE: fisheries come AFTER terminal detritus & import

% step 6b: define TargetGroup as PRODUCER to calculate Reach
TargetGroup_Reach                = CarniJelly_reach; % TargetGroup as PRODUCER to calculate Reach

% step 6c: calculate reach metrics ----------------------------------------
% returns:
%       TraceGroup_ReachVector	fraction of PRODUCER = TargetGroup_Reach production going to each CONSUMER
%                                       2 vertical vectors:
%                                           rows = producer groups
%                                           clm 1 = mean across MonteCarlo models
%                                           clm 2 = std across MonteCarlo models
%       TraceGroup_SystemReach  SYSTEM-LEVEL reach = ratio of total TraceGroup production going to all consumers over total production of all CONSUMER groups
%                                       row 1 = mean across MonteCarlo models
%                                       row 2 = std across MonteCarlo models
%       DietTrace_upward        fraction of energy within each linkage ultimately originating from PRODUCER = TraceGroup; mean across all MonteCarlo models
%                                       (2D matrix: rows of producers X clms of consumers)
%                                       (NOTE: use for web plotting reach arrow colors)
%       TraceFraction_upward    fraction of each CONSUMER production originating from PRODUCER = TraceGroup; mean across all MonteCarlo models
%                                       (horizontal vector: 1 X clms of consumers)
%                                       (NOTE: use for web plotting reach box colors)
%       DietTrace_downward      relative contribution of each linkage to production of CONSUMER = TraceGroup
%                                       (2D matrix: rows of producers X clms of consumers)
%                                       (NOTE: use for web plotting footprint arrow colors)
%       TraceFraction_downward  fraction of CONSUMER = TraceGroup production originating from each PRODUCER group; mean across all MonteCarlo models
%                                       (horizontal vector: 1 X clms of consumers)

[TraceGroup_ReachVector, TraceGroup_SystemReach, DietTrace_upward, TraceFraction_upward, DietTrace_downward, TraceFraction_downward] ...
        = f_Reach_01212018(EwEResult, MonteCarloStore, TargetGroup_Reach, use_MonteCarlo);





% *************************************************************************
% STEP 7: plot food web----------------------------------------------------
% step 7a: define box x-positions -----------------------------------------
%           NOTE: The order of x-position vector is the same as the ECOPATH (EwE) order
input_x = [
    1.75        % SmlPhyto
    4.25        % LargePhyto
    4           % microzooplankton
    2.5         % mesozooplankton
    5           % macrozooplankton
    0.75        % gelatinous zoop (herbivorous)
    1           % gelatinous zoop (carnivorous)
    6           % euphausiids
    4.5         % squid
    4           % planktivorous Fish
    6.5         % piscivorous Fish
    8           % benthivorous Fish
    3           % juvenileFish
    8.25        % deposit-Feeding Benthos
    7.5         % carnivorous Benthos
    6.75        % susp-Feeding Benthos
    3.75        % seabirds
    6           % graywhales
    3           % baleenWhales
    7           % pinnipeds
    8.5         % odontocetes
    0.5        % invert eggs
    0.5        % fish eggs
    7           % pelagic Detritus
    8.25        % benthic Detritus
    0.5         % ImportPrey
    4.5         % fisheries grp 1
    6           % fisheries grp 2
    ];


% step 10b: y-position (= TL) ---------------------------------------------
%           NOTE: uses EwE trophic levels, but you must add fisheries & import prey
TL_position                     = EwEResult.TL;
TL_position(end+1)              = 1.5; % Import (Term produced by ECOPATH)
TL_position((end+1):(end+2))    = EwEResult.TL_fisheries; % This is for the fisheries
TL_position(PlgcDetr_reach)     = 0.5; % shift for easier map reading
TL_position(BnthcDetr_reach)    = 0.5; % shift for easier map reading
TL_position(FishEggs_reach)     = 0.5; % shift for easier map reading






% step 10c: box labels ----------------------------------------------------
%           NOTE: you must add fisheries & import prey
GRPlabels                                   = EwEResult.AggLabel;
GRPlabels{end+1}                            = {'outside'; 'feeding'}; % Import (Term produced by ECOPATH)
GRPlabels{end+1}                            = 'fishery grp 1';          % This is for the fisheries 1
GRPlabels{end+1}                            = 'fishery grp 2';          % This is for the fisheries 2

GRPlabels{SmlPhyto_reach}                   = {'small'; 'phytoplankton'};
GRPlabels{LargePhyto_reach}                 = {'large'; 'phytoplankton'};
GRPlabels{MicroZ_reach}                     = {'micro-'; 'zooplankton'};
GRPlabels{MesoZ_reach}                      = {'meso-'; 'zooplankton'};
GRPlabels{MacroZ_reach}                     = {'macro-'; 'zooplankton'};
GRPlabels{HerbiJelly_reach}                 = {'gelatinous'; 'herbivores'};
GRPlabels{CarniJelly_reach}                 = {'gelatinous'; 'carnivores'};
GRPlabels{PlankFish_reach}                  = {'forage fish'};
GRPlabels{SuspFBen_reach}                   = {'suspension'; 'feeders'};
GRPlabels{DepFBen_reach}                    = {'deposit'; 'feeders'};
GRPlabels{SeaBird_reach}                    = {'seabirds'};
GRPlabels{BalWhal_reach}                    = {'baleen whales'};
GRPlabels{Pinniped_reach}                   = {'pinnipeds'};
GRPlabels{Odontocete_reach}                 = {'odontocetes'};
GRPlabels{PlgcDetr_reach}                   = {'pelagic'; 'detritus'};
GRPlabels{BnthcDetr_reach}                  = {'benthic'; 'detritus'};


% step 7d: plot web -------------------------------------------------------
p_WebPlotter_01032017(EwEResult, DietTrace_upward, TraceFraction_upward, DietTrace_downward, FootprintVector, GRPlabels, input_x, TL_position, 'off')

% print -dtiff -r300 'testtoss.tif'


% end m-file***************************************************************