function [TraceGroup_ReachVector, TraceGroup_SystemReach, DietTrace_upward, TraceFraction_upward, DietTrace_downward, TraceFraction_downward] ...
    = f_Reach_01212018(EwEResult, MonteCarloStore, TargetGroup_Reach, use_MonteCarlo)
% takes:
%       EwEResult
%       MonteCarloStore
%       TargetGroup_Reach
%       use_MonteCarlo              'y' or 'n'
% returns:
%       TraceGroup_ReachVector	fraction of PRODUCER = TargetGroup_Reach production going to each CONSUMER
%                                       2 vertical vectors:
%                                           rows = producer groups
%                                           clm 1 = mean across MonteCarlo models
%                                           clm 2 = std across MonteCarlo models
%       TraceGroup_SystemReach  SYSTEM-LEVEL reach = ratio of total TraceGroup production going to all consumers over total production of all CONSUMER groups
%                                       row 1 = mean across MonteCarlo models
%                                       row 2 = std across MonteCarlo models
%       DietTrace_upward        fraction of energy within each linkage ultimately originating from PRODUCER = TraceGroup; mean across all MonteCarlo models
%                                       (2D matrix: rows of producers X clms of consumers)
%                                       (NOTE: use for web plotting reach arrow colors)
%       TraceFraction_upward    fraction of each CONSUMER production originating from PRODUCER = TraceGroup; mean across all MonteCarlo models
%                                       (horizontal vector: 1 X clms of consumers)
%                                       (NOTE: use for web plotting reach box colors)
%       DietTrace_downward      relative contribution of each linkage to production of CONSUMER = TraceGroup
%                                       (2D matrix: rows of producers X clms of consumers)
%                                       (NOTE: use for web plotting footprint arrow colors)
%       TraceFraction_downward  fraction of CONSUMER = TraceGroup production originating from each PRODUCER group; mean across all MonteCarlo models
%                                       (horizontal vector: 1 X clms of consumers)
% calls:
%       f_DietTrace_03152015
%       f_DietTraceDownward_03152015
% revision date: 3-15-2015

% (3/15/15) bacteria are not considered to be consumers

% QQQ (3/15/15) there is an extra row in TraceGroup_ReachVector that I need to track down why it is there


% *************************************************************************
% STEP 1: unpack EwEResult-------------------------------------------------
num_fisheries = EwEResult.num_fisheries;




% STEP 1:
[diet_trace, trace_fraction] = f_DietTrace_03152015(EwEResult, MonteCarloStore, TargetGroup_Reach);
%   diet_trace     = fraction of energy within each linkge ultimately originating from PRODUCER = TraceGroup
%       (3D matrix: rows of producers X clms of consumers X num_trials)
%   trace_fraction = fraction of CONSUMER production originating from PRODUCER = TraceGroup
%       (1 X clms of consumers X num_trials)
% step 1b: take mean across MonteCarlo models
DietTrace_upward             = mean(diet_trace, 3); % mean across all MonteCarlo models
TraceFraction_upward         = mean(trace_fraction, 3); % mean across all MonteCarlo models
% (correct DietTrace_upward for mean values < 1 due to MonteCarlo models that lack some full cells found in original EwEResult.Diet)
looky_FullCell               = find(DietTrace_upward(TargetGroup_Reach, :) > 0);
DietTrace_upward(TargetGroup_Reach, looky_FullCell) = 1; % set TraceGroup row in DietTrace_upward to 1
clear looky_FullCell

% STEP 2:
[diet_trace_downward, trace_fraction_downward] = f_DietTraceDownward_03152015(EwEResult, MonteCarloStore, TargetGroup_Reach);
%   diet_trace_downward = relative contribution of each linkage to production of CONSUMER = TraceGroup; 
%       (3D matrix: rows of producers X clms of consumers X num_trials)
%   trace_fraction_downward = fraction of CONSUMER = TraceGroup production originating from each producer group
%       (clms of producers X 1 X num_trials)
% step 2b: take mean across MonteCarlo models
DietTrace_downward     = mean(diet_trace_downward, 3); % mean across all MonteCarlo models
TraceFraction_downward = mean(trace_fraction_downward, 3); % mean across all MonteCarlo models





% *************************************************************************
% STEP 3: 
% step 3a: get info from EwEResult and MonteCarloStore
if strcmp(use_MonteCarlo, 'y')
    [num_producers, num_consumers, num_trials]      = size(MonteCarloStore.diet); % (will get defined again below, but defining here allows footprint & reach to be calculated within separate m-files)
	trace_fraction                                  = squeeze(trace_fraction); % transpose
    trace_fraction(TargetGroup_Reach, 1:num_trials) = 0; % make reach on self = 0
    EwE_production                                  = MonteCarloStore.Biomass .* MonteCarloStore.PB; % (array of vertical vectors, consumers X trial_num)
    TempFisheries                                   = MonteCarloStore.Landings + MonteCarloStore.Discards;
else
    [num_producers, num_consumers, num_trials]      = size(EwEResult.Diet);
    num_trials                                      = 1; % (will get defined again below, but defining here allows footprint & reach to be calculated within separate m-files)
    trace_fraction(TargetGroup_Reach)               = 0; % make reach on self = 0
    EwE_production                                  = EwEResult.Production; % (vertical vector)
    TempFisheries                                   = EwEResult.Landings + EwEResult.Discards;
end % (strcmp(use_MonteCarlo, 'y'))

% step 3b: get addresses of nutrients, fisheries, BA & EM
EwE_GroupType                   = EwEResult.GroupType;
looky_NUTRIENTS                 = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_ANYNitroNutr);
looky_BA                        = find(EwE_GroupType        == EwEResult.GroupTypeDef_BA);
looky_EM                        = find(EwE_GroupType        == EwEResult.GroupTypeDef_EM);
looky_fisheries                 = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_fishery); % take out fisheries code here, it will be replaced in proper position in code below
EwE_GroupType([looky_NUTRIENTS' looky_BA' looky_EM' looky_fisheries']) = [];
EwE_GroupType(end+1)            = EwEResult.GroupTypeDef_import; % add import diet to EwE_GroupType vector QQQ make sure group type ored here is same as in f_DietTrace

% step 3c: prepare production terms
EwE_production(end+1, 1:num_trials) = 0; % add row to production for import prey
[Production_rows, Production_clms]  = size(EwE_production);
% add fisheries production
[fishery_rows, num_fisheries, trial_check] = size(TempFisheries); % number of separate fisheries
% this re-shaping routing necessary to de-confuse MonteCarlo models and different fishery types
if (num_fisheries == num_trials) && (trial_check == 1)
    num_fisheries = 1;
    Fisheries     = reshape(TempFisheries, fishery_rows, 1, num_trials); % reshape Footprint_vector to build up 3D Footprint_array
else
    Fisheries     = TempFisheries;    
end
fishery_size                                                                      = sum(Fisheries, 1); % size of each fishery (t)
fishery_size                                                                      = reshape(fishery_size, num_fisheries, num_trials); % reshape to append to bottom of EwE_production
EwE_production((Production_rows+1):(Production_rows+num_fisheries), 1:num_trials) = fishery_size;
Production_rows                                                                   = Production_rows + num_fisheries;
EwE_GroupType((end+1):(end+num_fisheries))                                        = EwEResult.GroupTypeDef_fishery; % add Fisheries to EwE_GroupType vector
% QQQ check that fisheries and import are being added in same order for footprint and reach





% *************************************************************************
% STEP 4: calculate TraceGroup reach in system-----------------------------
TraceGroup_ReachVector          = zeros((num_producers+num_fisheries), 2); % initialize
REACH                           = zeros((num_producers+num_fisheries), num_trials); % initialize

looky_consumer                  = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_ANYConsumer | floor(EwE_GroupType) == EwEResult.GroupTypeDef_fishery); % find consumer groups
looky_Micrograzers              = find(EwE_GroupType == 3.15 | EwE_GroupType == 3.25); % find pelagic and benthic micrograzer groups

total_production_CONSUMER                               = EwE_production;
total_production_CONSUMER(looky_Micrograzers, :)        = 0; % zero-out micrograzers :)); % total production of all CONSUMER groups (horizontal array of scalers; 1 X num_trials)
total_production_CONSUMER                               = sum(total_production_CONSUMER(looky_consumer, :)); % total production of all CONSUMER groups (horizontal array of scalers; 1 X num_trials)


if strcmp(use_MonteCarlo, 'y')
	TraceGroup_ReachProduction = trace_fraction'.* EwE_production'; % reach of TraceGroup to each consumer in terms of production; trace_fraction * (CONSUMER production) = (net PRODUCER production going to each CONSUMER) (array of horizontal vectors, num_trials X consumer grps)
else
    TraceGroup_ReachProduction = trace_fraction .* EwE_production'; % reach of TraceGroup to each consumer in terms of production; trace_fraction * (CONSUMER production) = (net PRODUCER production going to each CONSUMER) (array of horizontal vectors, num_trials X consumer grps)
end


total_TraceGroup_ReachProduction = sum(TraceGroup_ReachProduction'); % total TraceGroup production going to all consumers (horizontal array of scalers; 1 X num_trials)
temp_SystemReach                 = total_TraceGroup_ReachProduction ./ total_production_CONSUMER; % TraceGroup reach fraction across all consumer groups (horizontal array of scalers 1 X num_trials)
TraceGroup_SystemReach(1, 1)     = nanmean(temp_SystemReach);
TraceGroup_SystemReach(2, 1)     = nanstd(temp_SystemReach);

% calculate reach to each producer
for trial_loop = 1:num_trials
    REACH(1:Production_rows, trial_loop) = TraceGroup_ReachProduction(trial_loop, :)' / EwE_production(TargetGroup_Reach, trial_loop); % REACH: fraction of PRODUCER production going to CONSUMER
end

TraceGroup_ReachVector(1:(num_producers+num_fisheries), 1) = mean(REACH, 2);
TraceGroup_ReachVector(1:(num_producers+num_fisheries), 2) = std(REACH, [], 2);

% end m-file***************************************************************