function ECOTRAN_PEDIGREE = f_E2E_pedigree_08032018(EwEResult, PEDIGREE, ECOTRAN)
% by Jim Ruzicka
% calculate CV uncertainty terms to apply to ECOTRAN variables
% for use in Monte Carlo analyses with ECOTRAN variables (f_E2E_MonteCarlo_08032018)
%       taking advantage of fact that ECOTRAN models are by definition in balance
%       and avoiding the need to run Monte Carlo analyses at the ECOPATH (EwE) stage
% takes:
%       EwEResult
%       PEDIGREE
%       ECOTRAN
% calls:
%       none
% returns:
%       ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix         CV values for each element of the ECOTRAN EnergyBudget_matrix)
%       ECOTRAN_PEDIGREE.CV_ConsumptionBudget           CV values for ConsumptionBudget terms, by row:
%                                                                                       feces
%                                                                                       metabolism
%                                                                                       eggs
%                                                                                       predation
%                                                                                       senescence
%                                                                                       BA
%                                                                                       EM
%       ECOTRAN_PEDIGREE.ECTRN_ConsumptionBudget        ECOTRAN consumption budget (as fraction of total consumption), by row:
%                                                                                       feces
%                                                                                       metabolism
%                                                                                       eggs
%                                                                                       predation
%                                                                                       senescence
%                                                                                       BA
%                                                                                       EM
%       ECOTRAN_PEDIGREE.PEDIGREE_other                 "other" defined input pedigree terms for the whole model in general, pack these terms to pass along to other functions
%                                                                                       PEDIGREE_eggs
%                                                                                       PEDIGREE_predation
%                                                                                       PEDIGREE_senescence
%                                                                                       PEDIGREE_BacterialMTBLSM
%                                                                                       PEDIGREE_NutrientUptake
%
% FFF In future, could add uncertainty to detritus, metabolism fates, and retention scalers
% FFF in future, could add non-biological oxidation (reduction?) of NH4 to NO3
% QQQ EM needs to consider import diet info (can ignore for now since in all models they are small)
%
% revision date: 8-3-2018


% *************************************************************************
% STEP 1: read in PEDIGREE values------------------------------------------
% step 1a: read in parameter PEDIGREE values (CV) -------------------------
%          NOTE: (37 vertical vectors: EwE groups X 1)
% PEDIGREE_maxEE                      = PEDIGREE.PedigreeParameters(:, 1);    % max EE
PEDIGREE_Biomass                    = PEDIGREE.PedigreeParameters(:, 2);    % B (CV); (vertical vector: EwE grps X 1); EwE order
PEDIGREE_PB                         = PEDIGREE.PedigreeParameters(:, 3);    % PB (CV); (vertical vector: EwE grps X 1); EwE order
% PEDIGREE_QB                         = PEDIGREE.PedigreeParameters(:, 4);    % QB (CV); NOTE: recalculate as dependent variable PB/PQ
PEDIGREE_PQ                         = PEDIGREE.PedigreeParameters(:, 5);    % PQ (CV); (vertical vector: EwE grps X 1); EwE order
PEDIGREE_AE                         = PEDIGREE.PedigreeParameters(:, 6);    % AE (CV); (vertical vector: EwE grps X 1); EwE order
% PEDIGREE_MinBiomass                 = PEDIGREE.PedigreeParameters(:, 7);    % B (abs min)	
% PEDIGREE_MinPB                      = PEDIGREE.PedigreeParameters(:, 8);    % PB (abs min)	
% PEDIGREE_MinQB                      = PEDIGREE.PedigreeParameters(:, 9);    % QB (abs min); NOTE: recalculate as dependent variable
% PEDIGREE_MinPQ                      = PEDIGREE.PedigreeParameters(:, 10);   % PQ (abs min)	
% PEDIGREE_MinAE                      = PEDIGREE.PedigreeParameters(:, 11);   % AE (abs min)	
% PEDIGREE_MaxBiomass                 = PEDIGREE.PedigreeParameters(:, 12);   % B (abs max)	
% PEDIGREE_MaxPB                      = PEDIGREE.PedigreeParameters(:, 13);   % PB (abs max)
% PEDIGREE_MaxQB                      = PEDIGREE.PedigreeParameters(:, 14);   % QB (abs max); NOTE: recalculate as dependent variable
% PEDIGREE_MaxPQ                      = PEDIGREE.PedigreeParameters(:, 15);   % PQ (abs max)	
% PEDIGREE_MaxAE                      = PEDIGREE.PedigreeParameters(:, 16);   % AE (abs max)	
PEDIGREE_BA                         = PEDIGREE.PedigreeParameters(:, 17);   % BA (CV); (vertical vector: EwE grps X 1); EwE order
PEDIGREE_EM                         = PEDIGREE.PedigreeParameters(:, 18);   % EM (CV); (vertical vector: EwE grps X 1); EwE order
% PEDIGREE_EggRate                    = PEDIGREE.PedigreeParameters(:, 19);   % Egg rate (CV)	
% PEDIGREE_Metabolism                 = PEDIGREE.PedigreeParameters(:, 20);   % Metabolism rate (CV)	
% PEDIGREE_MinBA                      = PEDIGREE.PedigreeParameters(:, 21);   % BA (abs min)	
% PEDIGREE_MinEM                      = PEDIGREE.PedigreeParameters(:, 22);   % EM (abs min)	
% PEDIGREE_MinEggRate                 = PEDIGREE.PedigreeParameters(:, 23);   % Egg rate (abs min)	
% PEDIGREE_MinMetabolism              = PEDIGREE.PedigreeParameters(:, 24);   % Metabolism rate (abs min)	
% PEDIGREE_MaxBA                      = PEDIGREE.PedigreeParameters(:, 25);   % BA (abs max)	
% PEDIGREE_MaxEM                      = PEDIGREE.PedigreeParameters(:, 26);   % EM (abs max)	
% PEDIGREE_MaxEggRate                 = PEDIGREE.PedigreeParameters(:, 27);   % Egg rate (abs max)	
% PEDIGREE_MaxMetabolism              = PEDIGREE.PedigreeParameters(:, 28);   % Metabolism rate (abs max)	
% PEDIGREE_DetritusFate_feces         = PEDIGREE.PedigreeParameters(:, 29);   % DetritusFate_feces (CV)	
% PEDIGREE_DetritusFate_senescence    = PEDIGREE.PedigreeParameters(:, 30);   % DetritusFate_senescence (CV)
% PEDIGREE_ExcretionFate              = PEDIGREE.PedigreeParameters(:, 31);   % ExcretionFate (CV)	
% PEDIGREE_MinDetritusFate_feces      = PEDIGREE.PedigreeParameters(:, 32);   % DetritusFate_feces (min)	
% PEDIGREE_MinDetritusFate_senescence = PEDIGREE.PedigreeParameters(:, 33);   % DetritusFate_senescence (min)	
% PEDIGREE_MinExcretionFate           = PEDIGREE.PedigreeParameters(:, 34);   % ExcretionFate (min)	
% PEDIGREE_MaxDetritusFate_feces      = PEDIGREE.PedigreeParameters(:, 35);   % DetritusFate_feces (max)	
% PEDIGREE_MaxDetritusFate_senescence = PEDIGREE.PedigreeParameters(:, 36);   % DetritusFate_senescence (max)	
% PEDIGREE_MaxExcretionFate           = PEDIGREE.PedigreeParameters(:, 37);   % ExcretionFate (max)

% step 1b: read in fisheries PEDIGREE values (CV) -------------------------
PEDIGREE_Landings                   = PEDIGREE.PedigreeLandings;            % (2D matrix: EwE groups X fleets); EwE order
PEDIGREE_Discards                   = PEDIGREE.PedigreeDiscards;            % (2D matrix: EwE groups X fleets); EwE order

% step 1c: read in diet PEDIGREE values (CV) ------------------------------
PEDIGREE_Diet                       = PEDIGREE.PedigreeDiet(2:end, 2:end);  % trim off header row and column (this still has 1 row at bottom for import diet); (CV);
PEDIGREE_ImportDiet                 = PEDIGREE_Diet(end, :);                % save import diet; (horizontal vector: 1 X EwE grps); EwE order
PEDIGREE_Diet(end, :)               = [];                                   % remove import prey (applies only to seagulls & cormorants in NCC)

% % step 1d: read in pedigree scales ----------------------------------------
% %          NOTE: old method based on random EwE models
% STD_scaler                          = PEDIGREE.STD_scaler;                % STD range of acceptable drawing interval
% SamplingScale                       = PEDIGREE.SamplingScale;             % Bigger number makes STD range tighter; SamplingScale = 1 assumes the valid parameter interval represents +/- 2 STD_scaler (98% of values)
% SamplingScale_Diet                  = PEDIGREE.SamplingScale_Diet;
% SamplingScale_Fisheries             = PEDIGREE.SamplingScale_Fisheries;
% PEDIGREE_PhysiologyScaler           = PEDIGREE.PhysiologyScaler;
% PEDIGREE_BiomassScaler              = PEDIGREE.BiomassScaler;
% PEDIGREE_FisheriesScaler            = PEDIGREE.FisheriesScaler;           % (FFF change name to PEDIGREE.LandingsScaler)
% PEDIGREE_DiscardScaler              = PEDIGREE.DiscardScaler;
% PEDIGREE_MinBiomassFraction         = PEDIGREE.minBiomass;                % minimum fraction of original value
% PEDIGREE_MinPhysiologyFraction      = PEDIGREE.minPhysiology;             % minimum fraction of original value
% PEDIGREE_DietScaler                 = PEDIGREE.DietScaler;
% PEDIGREE_BAScaler                   = PEDIGREE.BAScaler;
% PEDIGREE_EMScaler                   = PEDIGREE.EMScaler;

% % step 1e: read in diet preference rules & guilds -----------------------
% %          NOTE: old method based on random EwE models
% PedigreeDietPrefRules               = PEDIGREE.PedigreeDietPrefRules(2:end, 2:end); % trim off header row and column (this still has 1 row at bottom for import diet)
% PreyGuild_name                      = PEDIGREE.PreyGuild_name;
% PreyGuild_code                      = PEDIGREE.PreyGuild_code;

% step 1f: defined pedigrees for all model groups -------------------------
PEDIGREE_eggs                       = PEDIGREE.PEDIGREE_eggs;               % egg pedigree for all groups (CV)
PEDIGREE_predation                  = PEDIGREE.PEDIGREE_predation;          % predation pedigree for all groups (CV)
PEDIGREE_senescence                 = PEDIGREE.PEDIGREE_senescence;         % senescence pedigree for all groups (CV)
PEDIGREE_BacterialMTBLSM            = PEDIGREE.PEDIGREE_BacterialMTBLSM;    % pedigree for implicit bacterial metabolism of terminal detritus (CV)
PEDIGREE_NutrientUptake             = PEDIGREE.PEDIGREE_NutrientUptake;     % pedigree for nutrient uptake by primary producers (CV)
PEDIGREE_other                      = [PEDIGREE_eggs; PEDIGREE_predation; PEDIGREE_senescence; ...
                                       PEDIGREE_BacterialMTBLSM; PEDIGREE_NutrientUptake]; % keep these "other" pedigree terms to pass along to other functions




                                   
% *************************************************************************
% STEP 2: read in EwEResult values-----------------------------------------
% step 2a: read in EwE addresses % sizes ----------------------------------
% NOTE: in EwE parameter order, includes fisheries at the end, 
%       detritus order as defined in original parameters, 
%       does not include nutrients at start
looky_ANYPrimaryProd                = find(floor(EwEResult.GroupType) == EwEResult.GroupTypeDef_ANYPrimaryProd);     % will need this info later
looky_fishery                       = find(floor(EwEResult.GroupType) == EwEResult.GroupTypeDef_fishery);
looky_DetritusAndEggs               = find(floor(EwEResult.GroupType) == EwEResult.GroupTypeDef_ANYDetritus | floor(EwEResult.GroupType) == EwEResult.GroupTypeDef_eggs); % will need this info later; (4-1-2014 includes eggs)
looky_detritus                      = find(floor(EwEResult.GroupType) == EwEResult.GroupTypeDef_ANYDetritus);        % all EwE detritus, excluding eggs
looky_PelagicDetritus               = find(EwEResult.GroupType        == EwEResult.GroupTypeDef_terminalPlgcDetr);   % terminal pelagic detritus group(s) in EwE
looky_BenthicDetritus               = find(EwEResult.GroupType        == EwEResult.GroupTypeDef_terminalBnthDetr);   % terminal benthic detritus group(s) in EwE
looky_eggs                          = find(EwEResult.GroupType        == EwEResult.GroupTypeDef_eggs);               % egg group(s) in EwE
looky_offal                         = find(EwEResult.GroupType        == EwEResult.GroupTypeDef_offal);              % offal group(s) in EwE
looky_OtherDetritus                 = find((floor(EwEResult.GroupType) == EwEResult.GroupTypeDef_ANYDetritus) & (EwEResult.GroupType ~= EwEResult.GroupTypeDef_terminalPlgcDetr) & (EwEResult.GroupType ~= EwEResult.GroupTypeDef_terminalBnthDetr)); % non-terminal, non-egg detritus groups
looky_DetritusInDetritus            = find((ismember(looky_DetritusAndEggs, (looky_DetritusAndEggs(~ismember(looky_DetritusAndEggs, looky_eggs))))));    % non-egg addresses in EwE_DetritusFate matrix
looky_EggsInDetritus                = find((ismember(looky_DetritusAndEggs, (looky_DetritusAndEggs(ismember(looky_DetritusAndEggs, looky_eggs))))));     % egg addresses in EwE_DetritusFate matrix

num_NitrogenNutrients               = EwEResult.num_NitrogenNutrients;
num_EwEgroups                       = length(EwEResult.GroupType);   % includes fisheries
num_fisheries                       = EwEResult.num_fisheries;
num_NonFisheryGroups                = num_EwEgroups - num_fisheries;
num_eggs                            = EwEResult.num_eggs;
num_detritus                        = EwEResult.num_detritus;
num_EggsAndDetritus                 = EwEResult.num_EggsAndDetritus;

ExcretionFate                       = EwEResult.ExcretionFate';              % flow to pelagic & benthic NH4 pools; (2D matrix: 2 X num_NonFisheryGroups)
DetritusFate_feces                  = EwEResult.DetritusFate_feces';         % flow to terminal pelagic & benthic detrituse pools; (2D matrix: 2 X num_NonFisheryGroups)
DetritusFate_senescence             = EwEResult.DetritusFate_senescence';    % flow to terminal pelagic & benthic detrituse pools; (2D matrix: 2 X num_NonFisheryGroups)
EwE_DetritusAndEggFate              = EwEResult.EwE_DetritusFate';           % (2D matrix: num_EggsAndDetritus X num_EwEgroups)
EwE_DetritusFate                    = EwE_DetritusAndEggFate(looky_DetritusInDetritus, :);
EwE_DetritusFate                    = EwE_DetritusFate ./ repmat(sum(EwE_DetritusFate), [num_detritus, 1]);  % remove eggs from detritus fate
EwE_DetritusFate(isnan(EwE_DetritusFate)) = 0;                               % convert div/0 NaNs to 0 
EwE_EggFate                         = EwE_DetritusAndEggFate(looky_EggsInDetritus, :);
EwE_EggFate                         = EwE_EggFate ./ repmat(sum(EwE_EggFate), [num_eggs, 1]);                % remove eggs from detritus fate
EwE_EggFate(isnan(EwE_EggFate))     = 0;                                     % convert div/0 NaNs to 0 

% step 2b: read in EwE parameter values -----------------------------------
%          NOTE: also remove import diet & import consumption
EwE_Biomass                         = EwEResult.Biomass;            % (vertical vector: num_NonFisheryGroups X 1)
EwE_PB                              = EwEResult.PB;                 % (vertical vector: num_NonFisheryGroups X 1)
EwE_QB                              = EwEResult.QB;                 % (vertical vector: num_NonFisheryGroups X 1)
EwE_PQ                              = EwEResult.PQ;                 % (vertical vector: num_NonFisheryGroups X 1)
EwE_AE                              = EwEResult.AE;                 % (vertical vector: num_NonFisheryGroups X 1)
EwE_BA                              = EwEResult.BA;                 % (t/km/y); (vertical vector: num_NonFisheryGroups X 1)
EwE_EM                              = EwEResult.EM;                 % immigration is NEGATIVE, emigration & fisheries are POSITIVE; (t/km/y); (vertical vector: num_NonFisheryGroups X 1)
EwE_EE                              = EwEResult.EwE_EE;             % (vertical vector: num_NonFisheryGroups X 1)
EwE_Diet                            = EwEResult.Diet(2:end, 2:end); % trim off header row and column (this still has 1 row at bottom for import diet); has cannibalism
EwE_ImportDiet                      = EwE_Diet(end, :);             % save import diet
EwE_Diet(end, :)                    = [];                           % remove import prey (applies only to seagulls & cormorants in NCC)
EwE_Consumption                     = EwEResult.Consumption;        % (no headers on this term); has cannibalism
EwE_ImportConsumption               = EwE_Consumption(end, :);      % retain import consumption (seagulls & cormorants in NCC)
EwE_Consumption(end, :)             = [];                           % remove import prey from consumption matrix

% step 2c: read in EwE fisheries values -----------------------------------
EwE_BiomassFishery                  = EwEResult.Biomass_fisheries;  % (t/km2); (vertical vector: num_fisheries X 1)
EwE_Landings                        = EwEResult.Landings;           % (2D matrix: num_NonFisheryGroups X num_fisheries)
EwE_Discards                        = EwEResult.Discards;           % (2D matrix: num_NonFisheryGroups X num_fisheries)
EwE_Catch                           = EwE_Landings + EwE_Discards;  % (2D matrix: num_NonFisheryGroups X num_fisheries)
EwE_TotalLandings                   = sum(EwE_Landings)';           % (vertical vector: num_fisheries X 1)
EwE_TotalDiscards                   = sum(EwE_Discards)';           % (vertical vector: num_fisheries X 1)
EwE_PB_fishery                      = EwEResult.PB_fisheries;       % (vertical vector: num_fisheries X 1)
EwE_QB_fishery                      = EwEResult.QB_fisheries;       % (vertical vector: num_fisheries X 1)
EwE_PQ_fishery                      = EwEResult.PQ_fisheries;       % (vertical vector: num_fisheries X 1)
EwE_AE_fishery                      = EwEResult.AE_fisheries;       % (vertical vector: num_fisheries X 1)
EwE_BA_fishery                      = EwEResult.BA_fisheries;       % (t/km2/y); (vertical vector: num_fisheries X 1)
EwE_EM_fishery                      = EwEResult.EM_fisheries;       % immigration is NEGATIVE, emigration & fisheries are POSITIVE; (t/km/y); (vertical vector: num_fisheries X 1)
EwE_EE_fishery                      = EwEResult.EwE_EE_fisheries;   % (vertical vector: num_fisheries X 1)

% other fishery terms...
DetritusFate_feces_fisheries        = EwEResult.DetritusFate_feces_fisheries';      % flow to terminal pelagic & benthic detritus pools; (2D matrix: 2 X num_fisheries)
DetritusFate_senescence_fisheries   = EwEResult.DetritusFate_senescence_fisheries'; % flow to terminal pelagic & benthic detritus pools; (2D matrix: 2 X num_fisheries)
ExcretionFate_fisheries             = EwEResult.ExcretionFate_fisheries';           % flow to pelagic & benthic NH4 pools; (2D matrix: 2 X num_fisheries)
% EwEResult.ProductionLossScaler_fisheries
% EwEResult.RetentionScaler_fisheries
% EwEResult.FunctionalResponseParams_fisheries
% EwEResult.EggProduction_forced_fisheries
% EwEResult.Metabolism_forced_fisheries





% *************************************************************************
% STEP 3: calculate pedigree terms-----------------------------------------
%
%           RULE: VAR     = STD^2 = (mean * CV)^2
%
%           RULE: CV      = STD / mean
%
%           RULE: adding & subtracting uncertainties: 
%                       mean(A+B) = mean(A) + mean(B) OR mean(A-B) = mean(A) - mean(B) 
%                       var(A+B)  = var(A) + var(B) (variances are additive whether adding or subtracting terms)
%
%           RULE: multiplying uncertainties of INDEPENDENT variables (i.e., we can ignore covariance term): 
%                 (from: https://chem.libretexts.org/Core/Analytical_Chemistry/Quantifying_Nature/Significant_Digits/Propagation_of_Error)
%                       mean(A*B) = mean(A) * mean(B)
%                       var(A*B)  = [mean(A*B)^2] * [(var(A)/mean(A)^2) + (var(B)/mean(B)^2)]    NOTE: Goodman (1960) derives the same equation but also adds the term [... + var(A)*var(B)]
%                                                                                                      (Goodman, L.A. 1960. On the exact variance of products. Journal of the American Statistical Association. December 1960, 708?713. DOI: 10.2307/2281592)
%           RULE: dividing uncertainties of INDEPENDENT variables (i.e., we ignore covariance term):
%                 (from: Seltman, H. Approximations for Mean and Variance of a Ratio. Carnegie Mellon University)
%                       mean(A/B) = mean(A) / mean(B) 
%                       var(A/B)  = [mean(A)^2/mean(B)^2] * [(var(A)/mean(A)^2) + (var(B)/mean(B)^2)]

% step 3a: convert CV values to variance values ---------------------------
VAR_Biomass                     = (EwE_Biomass    .* PEDIGREE_Biomass).^2;     	% variance; (vertical vector: num_NonFisheryGroups X 1)
VAR_PB                          = (EwE_PB         .* PEDIGREE_PB).^2;          	% variance; (vertical vector: num_NonFisheryGroups X 1)
% VAR_QB                          = (EwE_QB         .* PEDIGREE_QB).^2;        	  % variance; (vertical vector: num_NonFisheryGroups X 1); NOTE: recalculate as dependent variable
VAR_PQ                          = (EwE_PQ         .* PEDIGREE_PQ).^2;         	% variance; (vertical vector: num_NonFisheryGroups X 1)
VAR_AE                          = (EwE_AE         .* PEDIGREE_AE).^2;          	% variance; (vertical vector: num_NonFisheryGroups X 1)
VAR_BA                          = (EwE_BA         .* PEDIGREE_BA).^2;          	% variance; (vertical vector: num_NonFisheryGroups X 1)
VAR_EM                          = (EwE_EM         .* PEDIGREE_EM).^2;          	% variance; (vertical vector: num_NonFisheryGroups X 1)
VAR_Diet                        = (EwE_Diet       .* PEDIGREE_Diet).^2;       	% variance; (matrix)
VAR_ImportDiet                  = (EwE_ImportDiet .* PEDIGREE_ImportDiet).^2;   % variance; (horizontal vector: 1 X num_NonFisheryGroups)

% VAR_EggRate                     = (EwE_FFF .* PEDIGREE_EggRate).^2;                   % variance;	
% VAR_Metabolism                  = (EwE_FFF .* PEDIGREE_Metabolism).^2;                % variance;
% VAR_DetritusFate_feces          = (EwE_FFF .* PEDIGREE_DetritusFate_feces).^2;        % variance;	
% VAR_DetritusFate_senescence     = (EwE_FFF .* PEDIGREE_DetritusFate_senescence).^2;   % variance;
% VAR_ExcretionFate               = (EwE_FFF .* PEDIGREE_ExcretionFate).^2;             % variance;

% step 3b: prep fisheries pedigree ----------------------------------------
VAR_Landings                        = (EwE_Landings .* PEDIGREE_Landings).^2;       % variance; (2D matrix: num_NonFisheryGroups X num_fisheries)
VAR_Discards                        = (EwE_Discards .* PEDIGREE_Discards).^2;       % variance; (2D matrix: num_NonFisheryGroups X num_fisheries)
VAR_TotalLandings                   = sum(VAR_Landings)';                           % variance; (vertical vector: num_fisheries X 1)
VAR_TotalDiscards                   = sum(VAR_Discards)';                           % variance; (vertical vector: num_fisheries X 1)
VAR_BiomassFishery                  = VAR_TotalLandings + VAR_TotalDiscards;        % variance; (vertical vector: num_fisheries X 1)
VAR_PBfishery                       = ((EwE_TotalLandings.^2)./(EwE_BiomassFishery.^2)) .* ...
                                      ((VAR_TotalLandings./(EwE_TotalLandings.^2)) + (VAR_BiomassFishery./(EwE_BiomassFishery.^2))); % variance; (vertical vector: num_fisheries X 1); PB_fisheries = landings / (landings + discards)
VAR_PBfishery(isnan(VAR_PBfishery))	= 0;                                            % clean div/0 NaNs
VAR_QBfishery                       = zeros(num_fisheries, 1);                      % variance; (vertical vector: num_fisheries X 1)
VAR_PQfishery                       = VAR_PBfishery;                                % variance; (vertical vector: num_fisheries X 1)
VAR_AEfishery                       = VAR_PBfishery;                                % variance; (vertical vector: num_fisheries X 1)
VAR_BAfishery                       = zeros(num_fisheries, 1);                      % variance; (vertical vector: num_fisheries X 1)
VAR_EMfishery                       = VAR_TotalLandings;                            % variance; (vertical vector: num_fisheries X 1)

VAR_Catch                           = VAR_Landings + VAR_Discards;                  % variance; (2D matrix: num_NonFisheryGroups X num_fisheries)
repmat_VAR_BiomassFishery           = VAR_BiomassFishery';                          % variance; (horizontal vector: 1 X num_fisheries)
repmat_VAR_BiomassFishery           = repmat(repmat_VAR_BiomassFishery, [num_NonFisheryGroups, 1]); % variance; (2D matrix: num_NonFisheryGroups X num_fisheries)

repmat_EwE_BiomassFishery           = EwE_BiomassFishery';                          % (horizontal vector: 1 X num_fisheries)
repmat_EwE_BiomassFishery           = repmat(repmat_EwE_BiomassFishery, [num_NonFisheryGroups, 1]); % variance; (2D matrix: num_NonFisheryGroups X num_fisheries)
VAR_Catch                           = ((EwE_Catch.^2)./(repmat_EwE_BiomassFishery.^2)) .* ...
                                      ((VAR_Catch./(EwE_Catch.^2)) + (repmat_VAR_BiomassFishery./(repmat_EwE_BiomassFishery.^2))); % variance; (vertical vector: num_fisheries X 1); PB_fisheries = landings / (landings + discards)
VAR_Catch(isnan(VAR_Catch))         = 0;                                            % clean div/0 NaNs





% *************************************************************************
% STEP 4: append fishery terms---------------------------------------------
% step 4a: appending fishery values at end of mean parameter terms --------
EwE_Biomass                    = [EwE_Biomass; EwE_BiomassFishery];	% (vertical vector: num_EwEgroups X 1)
EwE_PB                         = [EwE_PB;      EwE_PB_fishery];        	% (vertical vector: num_EwEgroups X 1)
EwE_QB                         = [EwE_QB;      EwE_QB_fishery];         % (vertical vector: num_EwEgroups X 1)
EwE_PQ                         = [EwE_PQ;      EwE_PQ_fishery];         % (vertical vector: num_EwEgroups X 1)
EwE_AE                         = [EwE_AE;      EwE_AE_fishery];         % (vertical vector: num_EwEgroups X 1)
EwE_BA                         = [EwE_BA;      EwE_BA_fishery];         % (t/km/y) (vertical vector: num_EwEgroups X 1)
EwE_EM                         = [EwE_EM;      EwE_EM_fishery];         % immigration is NEGATIVE, emigration & fisheries are POSITIVE; (t/km/y); (vertical vector: num_EwEgroups X 1)
EwE_EM(looky_DetritusAndEggs)  = 0;                                     % set all detritus EM to 0
EwE_EE                         = [EwE_EE;      EwE_EE_fishery];         % immigration is NEGATIVE, emigration & fisheries are POSITIVE; (t/km/y); (vertical vector: num_EwEgroups X 1)
EwE_MTBLSM                     = EwE_AE - EwE_PQ;                       % fraction of consumption going to metabolism (NH4 production); Metabolism = 1 - PQ - feces = (1 - (EwE_PQ + (1-EwE_AE))); (vertical vector: num_EwEgroups X 1)

% step 4b: append fishery values to diet & consumption values -------------
EwE_Diet(:, (end+1):(end+num_fisheries))              = EwE_Catch;
EwE_Diet((end+1):(end+num_fisheries), :)              = 0;                    	% (2D matrix: num_EwEgroups X num_EwEgroups)
EwE_ImportDiet(1, (end+1):(end+num_fisheries))        = 0;                     	% append import fisheries as 0; (horizontal vector: 1 X num_EwEgroups)
EwE_Consumption(:, (end+1):(end+num_fisheries))       = EwE_Catch;              % (2D matrix: num_EwEgroups X num_EwEgroups)
EwE_Consumption((end+1):(end+num_fisheries), :)       = 0;                     	% (2D matrix: num_EwEgroups X num_EwEgroups)
EwE_ImportConsumption(1, (end+1):(end+num_fisheries)) = 0;                   	% append import fisheries as 0; (horizontal vector: 1 X num_EwEgroups)

[Diet_rows, Diet_clms]                                = size(EwE_Diet);
sum_Diet                                              = repmat(sum(EwE_Diet), [Diet_rows, 1]);
EwE_Diet                                              = EwE_Diet ./ sum_Diet;	% rescale diet clms to sum to 1; This makes fishery column into fractions of total fishery and corrects for removing the trace of import diet
EwE_Diet(isnan(EwE_Diet))                             = 0;                    	% convert NaNs (phyto & detritus groups) to 0 

% step 4c: append fishery values to variance terms ------------------------
VAR_Biomass                                     = [VAR_Biomass; VAR_BiomassFishery];    % (vertical vector: num_EwEgroups X 1)
VAR_PB                                          = [VAR_PB;      VAR_PBfishery];         % (vertical vector: num_EwEgroups X 1)
% VAR_QB                                          = [VAR_QB;      VAR_QBfishery];         % (vertical vector: num_EwEgroups X 1)
VAR_PQ                                          = [VAR_PQ;      VAR_PQfishery];         % (vertical vector: num_EwEgroups X 1)
VAR_AE                                          = [VAR_AE;      VAR_AEfishery];         % (vertical vector: num_EwEgroups X 1)
VAR_BA                                          = [VAR_BA;      VAR_BAfishery];         % (vertical vector: num_EwEgroups X 1)
VAR_EM                                          = [VAR_EM;      VAR_EMfishery];         % (vertical vector: num_EwEgroups X 1)
VAR_Diet(:, (end+1):(end+num_fisheries))        = VAR_Catch;
VAR_Diet((end+1):(end+num_fisheries), :)        = 0;                                    % (2D matrix: num_EwEgroups X num_EwEgroups)
VAR_ImportDiet(1, (end+1):(end+num_fisheries))  = 0;                                    % append import fisheries as 0; (horizontal vector: 1 X num_EwEgroups)

% step 4d: append fishery values to fate terms ----------------------------
ExcretionFate                                   = [ExcretionFate           ExcretionFate_fisheries];            % (2D matrix: 2 X num_EwEgroups)
DetritusFate_feces                              = [DetritusFate_feces      DetritusFate_feces_fisheries];       % (2D matrix: 2 X num_EwEgroups)
DetritusFate_senescence                         = [DetritusFate_senescence DetritusFate_senescence_fisheries];	% (2D matrix: 2 X num_EwEgroups)





% *************************************************************************
% STEP 5: calculate derived uncertainty terms------------------------------
% step 5a: derived QB (QB = PB/PQ) ----------------------------------------
VAR_QB                  = ((EwE_PB.^2)./(EwE_PQ.^2)) .* ...
                          ((VAR_PB./(EwE_PB.^2)) + (VAR_PQ./(EwE_PQ.^2))); % variance; (vertical vector: num_EwEgroups X 1)
VAR_QB(looky_fishery)   = 0;                               	               % QB for fisheries is always 1 beacause Q = B, so VAR_QB for fisheries = 0; (vertical vector: num_EwEgroups X 1)

% step 5b: derived MEAN total consumption and VARIANCE total consumption --
%          NOTE: q_c = B_c * (Q/B)_c
%          NOTE: alternative formula: q = (production + metablism) / assimiation efficiency
EwE_q                   = EwE_Biomass .* EwE_QB;                                     % total consumption rate q; (vertical vector: num_EwEgroups X 1)
VAR_q                   = (EwE_Biomass.^2 .* EwE_QB.^2) .* ...
                          ((VAR_Biomass./(EwE_Biomass.^2)) + (VAR_QB./(EwE_QB.^2))); % variance; (vertical vector: num_EwEgroups X 1)

% step 5c: consider inner-domain consumption and import diet consumption separately
%          NOTE: import consumption vector Qimport_pc = Dimport_pc * q_c
EwE_q                   = EwE_q';                           % transpose; total consumption rate vector; (horizontal vector: 1 X num_EwEgroups)
EwE_q_import            = EwE_ImportDiet .* EwE_q;          % import consumtion rate vector Qimport_pc; (horizontal vector: 1 X num_EwEgroups)
EwE_q_internal          = EwE_q - EwE_q_import;             % subtract import diet consumption OUT of consumption matrix; (horizontal vector: 1 X num_EwEgroups)

VAR_q                   = VAR_q';                           % transpose; (horizontal vector: 1 X num_EwEgroups)
VAR_q_import            = (EwE_ImportDiet.^2 .* EwE_q.^2) .* ...
                          ((VAR_ImportDiet./(EwE_ImportDiet.^2)) + (VAR_q./(EwE_q.^2))); % variance; (horizontal vector: 1 X num_EwEgroups); 
                                                                                         % note use of EwE_q here rather than EwE_q_import (because import diet represents proportions of total consumption EwE_q)
VAR_q_import(isnan(VAR_q_import))	= 0;                 	% clean div/0 NaNs
VAR_q_internal          = VAR_q + VAR_q_import;             % variance of consumption exclusively WITHIN model domain; (horizontal vector: 1 X num_EwEgroups)

% step 5d: derived consumption matrix VARIANCE ----------------------------
%          NOTE: consumtion matrix Q_pc = D_pc * q_c
repmat_EwE_q_internal   = repmat(EwE_q_internal, [Diet_rows, 1]);    % (2D matrix: num_EwEgroups X num_EwEgroups)
EwE_Q                   = EwE_Diet .* repmat_EwE_q_internal;         % consumtion matrix Q_pc; (2D matrix: num_EwEgroups X num_EwEgroups); 
                                                                     % note use of repmat_EwE_q_internal here rather than EwE_q (because diet matrix has been normalized to 1 after import diet removed)

repmat_VAR_q_internal   = repmat(VAR_q_internal, [Diet_rows, 1]);    % (2D matrix: num_EwEgroups X num_EwEgroups)
VAR_Q                   = (EwE_Diet.^2 .* repmat_EwE_q_internal.^2) .* ...
                          ((VAR_Diet./(EwE_Diet.^2)) + (repmat_VAR_q_internal./(repmat_EwE_q_internal.^2))); % variance; (vertical vector: num_EwEgroups X 1)
VAR_Q(isnan(VAR_Q))     = 0;                                         % clean div/0 NaNs

% step 5e: derived predation vector VARIANCE ------------------------------
%          NOTE: M2 = sigma_c(D_pc * q_c)
EwE_M2                  = sum(EwE_Q, 2);                    % total predation on each p; (vertical vector: num_EwEgroups X 1)
repmat_EwE_M2           = repmat(EwE_M2, [1, Diet_clms]);   % (2D matrix: num_EwEgroups X num_EwEgroups)

VAR_M2                  = sum(VAR_Q, 2);                    % (vertical vector)
repmat_VAR_M2           = repmat(VAR_M2, [1, Diet_clms]);   % (2D matrix: num_EwEgroups X num_EwEgroups)

% step 5e: derived predation matrix VARIANCE ------------------------------
%          NOTE: A_cp = Q_cp / M2_p
%          NOTE: PredationBudget: for each producer, p, and consumer, c: ((b_pc * Q_c) / M2_p)) = the fraction of total predation on each producer, p, going to each consumer, c.
EwE_A                   = EwE_Q ./ repmat_EwE_M2;           % predation matrix; (2D matrix: num_EwEgroups X num_EwEgroups)
EwE_A(isnan(EwE_A))     = 0;                                % replace div/0 NaN values with 0; these are top predators and fisheries

VAR_A                   = ((EwE_Q.^2)./(repmat_EwE_M2.^2)) .* ...
                          ((VAR_Q./(EwE_Q.^2)) + (repmat_VAR_M2./(repmat_EwE_M2.^2))); % variance; (2D matrix: num_EwEgroups X num_EwEgroups)
VAR_A(isnan(VAR_A))     = 0;                                % replace div/0 NaN values with 0

EwE_A                       = EwE_A';                           % transpose to A_cp orientation; (2D matrix: num_EwEgroups X num_EwEgroups)
VAR_A                       = VAR_A';                           % transpose to A_cp orientation; (2D matrix: num_EwEgroups X num_EwEgroups)

% step 5f: derived metabolism VARIANCE ------------------------------------
%          NOTE: EwE_MTBLSM = EwE_AE - EwE_PQ
EwE_MTBLSM                  = EwE_AE - EwE_PQ;                  % (vertical vector: num_EwEgroups X 1)
VAR_MTBLSM                  = VAR_AE + VAR_PQ;                  % (vertical vector: num_EwEgroups X 1)
VAR_MTBLSM(looky_fishery)   = 0;                                % fishery metabolism always = 0 

% step 5g: derived feces VARIANCE -----------------------------------------
EwE_feces                   = 1 - EwE_AE;                       % (vertical vector: num_EwEgroups X 1)
VAR_feces                   = VAR_AE;                           % (vertical vector: num_EwEgroups X 1)

% step 5h: expand defined CV terms ----------------------------------------
CV_eggs                     = ones(1, num_EwEgroups) * PEDIGREE_eggs;       % (horizontal vector: 1 X num_EwEgroups)
CV_predation                = ones(1, num_EwEgroups) * PEDIGREE_predation;  % (horizontal vector: 1 X num_EwEgroups)
CV_senescence               = ones(1, num_EwEgroups) * PEDIGREE_senescence;	% (horizontal vector: 1 X num_EwEgroups)

% step 5i: add import consumption to EM and express BA & EM as consumption rate terms
%          NOTE: EM at this point includes import diet and fishery landings
%          NOTE: EwE_BA and EwE_EM are in expressed in terms of production, convert them to consumption terms
EwE_BA                    	= EwE_BA ./ EwE_PQ;                                 % (t/km2/y); (vertical vector: num_EwEgroups X 1)
EwE_EM                    	= EwE_EM ./ EwE_PQ;                                 % (t/km2/y); (vertical vector: num_EwEgroups X 1)

VAR_BA                      = ((EwE_BA.^2)./(EwE_PQ.^2)) .* ...
                              ((VAR_BA./(EwE_BA.^2)) + (VAR_PQ./(EwE_PQ.^2)));	% variance; (vertical vector: num_EwEgroups X 1)
VAR_BA(isnan(VAR_BA))       = 0;                                                % replace div/0 NaN values with 0

EwE_q_import                = EwE_q_import';                                    % transpose; (vertical vector: num_EwEgroups X 1)
VAR_q_import                = VAR_q_import';                                    % transpose; (vertical vector: num_EwEgroups X 1)
VAR_EM                      = (((EwE_EM+EwE_q_import).^2)./(EwE_PQ.^2)) .* ...  % correct VAR_EM after re-expression to consumption terms; note addition of EwE_q_import to take out import diet from EwE_EM (until calcs just below)
                              ((VAR_EM./((EwE_EM+EwE_q_import).^2)) + (VAR_PQ./(EwE_PQ.^2)));	% variance; (vertical vector: num_EwEgroups X 1)
VAR_EM(isnan(VAR_EM))       = 0;                                                % replace div/0 NaN values with 0
VAR_EM                      = VAR_EM + VAR_q_import;                            % NOTE: VAR_EM as defined in the excel model pedigree refers to migration uncertainty.
                                                                                %       We must add the uncertainty due to import diet consumption
                                                                                %       (vertical vector: num_EwEgroups X 1)





% *************************************************************************
% STEP 6: express uncertainties as CV terms -------------------------------
CV_Biomass                      = (sqrt(VAR_Biomass) ./ EwE_Biomass)';	% CV; (horizontal vector: 1 X num_EwEgroups)
CV_PB                           = (sqrt(VAR_PB)      ./ EwE_PB)';       % CV; (horizontal vector: 1 X num_EwEgroups)
CV_QB                           = (sqrt(VAR_QB)      ./ EwE_QB)';       % CV; (horizontal vector: 1 X num_EwEgroups)
CV_PQ                           = (sqrt(VAR_PQ)      ./ EwE_PQ)';       % CV; (horizontal vector: 1 X num_EwEgroups)
CV_AE                           = (sqrt(VAR_AE)      ./ EwE_AE)';       % CV; (horizontal vector: 1 X num_EwEgroups)
CV_BA                           = (sqrt(VAR_BA)      ./ EwE_BA)';       % CV; (horizontal vector: 1 X num_EwEgroups)
CV_EM                           = (sqrt(VAR_EM)      ./ EwE_EM)';       % CV; (horizontal vector: 1 X num_EwEgroups)
CV_MTBLSM                       = (sqrt(VAR_MTBLSM)  ./ EwE_MTBLSM)';   % CV; (horizontal vector: 1 X num_EwEgroups)
CV_A                            = sqrt(VAR_A)        ./ EwE_A;          % CV; (2D matrix: num_EwEgroups X num_EwEgroups)
CV_feces                        = (sqrt(VAR_feces)   ./ EwE_feces)';    % CV; (2D matrix: num_EwEgroups X num_EwEgroups)

CV_Biomass(isnan(CV_Biomass))   = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_PB(isnan(CV_PB))             = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_QB(isnan(CV_QB))             = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_PQ(isnan(CV_PQ))             = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_AE(isnan(CV_AE))             = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_BA(isnan(CV_BA))             = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_EM(isnan(CV_EM))             = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_MTBLSM(isnan(CV_MTBLSM))     = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_MTBLSM(isinf(CV_MTBLSM))     = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_A(isnan(CV_A))               = 0; % replace div/0 NaN values with 0; (2D matrix: num_EwEgroups X num_EwEgroups)
CV_feces(isnan(CV_feces))       = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)
CV_feces(isinf(CV_feces))       = 0; % replace div/0 NaN values with 0; (horizontal vector: 1 X num_EwEgroups)





% *************************************************************************
% STEP 7: put means & uncertainties into ECOTRAN order --------------------
% step 7a: define nutrient rows, clms, & cells ----------------------------
Nutrient_rows                   = zeros(num_NitrogenNutrients, num_EwEgroups);
Nutrient_clms                   = zeros((num_NitrogenNutrients + num_EwEgroups), num_NitrogenNutrients);
Nutrient_cells                  = zeros(1, num_NitrogenNutrients); % (horizontal vector: 1 X num_NitrogenNutrients)

% step 7b: arrange vector parameters --------------------------------------
TerminalDetritus_CV_Biomass                                     = CV_Biomass(1,    [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_PB                                          = CV_PB(1,         [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_QB                                          = CV_QB(1,         [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_PQ                                          = CV_PQ(1,         [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_AE                                          = CV_AE(1,         [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_BA                                          = CV_BA(1,         [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_EM                                          = CV_EM(1,         [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_MTBLSM                                      = CV_MTBLSM(1,     [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_feces                                       = CV_feces(1,      [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_eggs                                        = CV_eggs(1,       [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_predation                                   = CV_predation(1,  [looky_PelagicDetritus; looky_BenthicDetritus]);
TerminalDetritus_CV_senescence                                  = CV_senescence(1, [looky_PelagicDetritus; looky_BenthicDetritus]);

CV_Biomass([looky_PelagicDetritus; looky_BenthicDetritus])      = []; % delete terminal detritus clms
CV_PB([looky_PelagicDetritus; looky_BenthicDetritus])           = []; % delete terminal detritus clms
CV_QB([looky_PelagicDetritus; looky_BenthicDetritus])           = []; % delete terminal detritus clms
CV_PQ([looky_PelagicDetritus; looky_BenthicDetritus])           = []; % delete terminal detritus clms
CV_AE([looky_PelagicDetritus; looky_BenthicDetritus])           = []; % delete terminal detritus clms
CV_BA([looky_PelagicDetritus; looky_BenthicDetritus])           = []; % delete terminal detritus clms
CV_EM([looky_PelagicDetritus; looky_BenthicDetritus])           = []; % delete terminal detritus clms
CV_MTBLSM([looky_PelagicDetritus; looky_BenthicDetritus])       = []; % delete terminal detritus clms
CV_feces([looky_PelagicDetritus; looky_BenthicDetritus])        = []; % delete terminal detritus clms
CV_eggs([looky_PelagicDetritus; looky_BenthicDetritus])         = []; % delete terminal detritus clms
CV_predation([looky_PelagicDetritus; looky_BenthicDetritus])    = []; % delete terminal detritus clms
CV_senescence([looky_PelagicDetritus; looky_BenthicDetritus])   = []; % delete terminal detritus clms

CV_Biomass(end+1:end+2)                                         = TerminalDetritus_CV_Biomass;      % append terminal detritus to end clms
CV_PB(end+1:end+2)                                              = TerminalDetritus_CV_PB;           % append terminal detritus to end clms
CV_QB(end+1:end+2)                                              = TerminalDetritus_CV_QB;           % append terminal detritus to end clms
CV_PQ(end+1:end+2)                                              = TerminalDetritus_CV_PQ;           % append terminal detritus to end clms
CV_AE(end+1:end+2)                                              = TerminalDetritus_CV_AE;           % append terminal detritus to end clms
CV_BA(end+1:end+2)                                              = TerminalDetritus_CV_BA;           % append terminal detritus to end clms
CV_EM(end+1:end+2)                                              = TerminalDetritus_CV_EM;           % append terminal detritus to end clms
CV_MTBLSM(end+1:end+2)                                          = TerminalDetritus_CV_MTBLSM;       % append terminal detritus to end clms
CV_feces(end+1:end+2)                                           = TerminalDetritus_CV_feces;        % append terminal detritus to end clms
CV_eggs(end+1:end+2)                                            = TerminalDetritus_CV_eggs;         % append terminal detritus to end clms
CV_predation(end+1:end+2)                                       = TerminalDetritus_CV_predation;    % append terminal detritus to end clms
CV_senescence(end+1:end+2)                                      = TerminalDetritus_CV_senescence;	% append terminal detritus to end clms

CV_Biomass                                                      = [Nutrient_cells CV_Biomass];      % add clms for nutrients; nutrients MUST be in first clms
CV_PB                                                           = [Nutrient_cells CV_PB];           % add clms for nutrients; nutrients MUST be in first clms
CV_QB                                                           = [Nutrient_cells CV_QB];           % add clms for nutrients; nutrients MUST be in first clms
CV_PQ                                                           = [Nutrient_cells CV_PQ];           % add clms for nutrients; nutrients MUST be in first clms
CV_AE                                                           = [Nutrient_cells CV_AE];           % add clms for nutrients; nutrients MUST be in first clms
CV_BA                                                           = [Nutrient_cells CV_BA];           % add clms for nutrients; nutrients MUST be in first clms
CV_EM                                                           = [Nutrient_cells CV_EM];           % add clms for nutrients; nutrients MUST be in first clms
CV_MTBLSM                                                       = [Nutrient_cells CV_MTBLSM];       % add clms for nutrients; nutrients MUST be in first clms
CV_feces                                                        = [Nutrient_cells CV_feces];        % add clms for nutrients; nutrients MUST be in first clms
CV_eggs                                                         = [Nutrient_cells CV_eggs];         % add clms for nutrients; nutrients MUST be in first clms
CV_predation                                                    = [Nutrient_cells CV_predation];    % add clms for nutrients; nutrients MUST be in first clms
CV_senescence                                                   = [Nutrient_cells CV_senescence];	% add clms for nutrients; nutrients MUST be in first clms

% step 7c: arrange matrix parameters --------------------------------------
% move rows
TerminalDetritus_CV_A                                   = CV_A([looky_PelagicDetritus; looky_BenthicDetritus], :);
CV_A([looky_PelagicDetritus; looky_BenthicDetritus], :) = [];                       % delete terminal detritus rows
CV_A(end+1:end+2, :)                                    = TerminalDetritus_CV_A;    % append terminal detritus to end rows
% move clms
TerminalDetritus_CV_A                                   = CV_A(:, [looky_PelagicDetritus; looky_BenthicDetritus]);
CV_A(:, [looky_PelagicDetritus; looky_BenthicDetritus]) = [];                       % delete terminal detritus clms
CV_A(:, end+1:end+2)                                    = TerminalDetritus_CV_A;    % append terminal detritus to end rows
CV_A                                                    = [Nutrient_rows; CV_A];    % add rows for nutrients; nutrients MUST be in first rows
CV_A                                                    = [Nutrient_clms CV_A];     % add clms for nutrients; nutrients MUST be in first clms





% *************************************************************************
% STEP 8: load & prep ECOTRAN terms----------------------------------------
% step 8a: ECOTRAN group count --------------------------------------------
ECTRN_EnergyBudget_matrix       = ECOTRAN.EnergyBudget_matrix; % (matrix: num_ECTRNgroups X num_ECTRNgroups)
[num_ECTRNgroups toss]          = size(ECTRN_EnergyBudget_matrix);

% step 8b: ECOTRAN parameter vectors --------------------------------------
ECTRN_Biomass                   = ECOTRAN.EwE_Biomass';	% (horizontal vector: 1 X num_ECTRNgroups)
ECTRN_PB                        = ECOTRAN.EwE_PB';      % (horizontal vector: 1 X num_ECTRNgroups)
ECTRN_QB                        = ECOTRAN.EwE_QB';      % (horizontal vector: 1 X num_ECTRNgroups)
ECTRN_PQ                        = ECOTRAN.EwE_PQ';      % (horizontal vector: 1 X num_ECTRNgroups)
ECTRN_AE                        = ECOTRAN.EwE_AE';      % (horizontal vector: 1 X num_ECTRNgroups)
ECTRN_BA                        = ECOTRAN.EwE_BA';      % (horizontal vector: 1 X num_ECTRNgroups)
ECTRN_EM                        = ECOTRAN.EwE_EM';      % (horizontal vector: 1 X num_ECTRNgroups)

% step 8c: ECOTRAN consumption budget terms -------------------------------
ECTRN_BioBudget                 = ECOTRAN.BioenergeticBudget;                   % (2D matrix: 3 X num_ECTRNgroups); [production; feces; metabolism]
ECTRN_BioBudget_production      = ECOTRAN.BioenergeticBudget_ProductionDetail;  % (2D matrix: 5 X num_ECTRNgroups); [predation; eggs; senescence; BA; EM]
ECTRN_BioBudget_feces           = ECTRN_BioBudget(2, :);
ECTRN_BioBudget_MTBLSM          = ECTRN_BioBudget(3, :);
ECTRN_BioBudget_eggs            = ECTRN_BioBudget_production(2, :);
ECTRN_BioBudget_predation       = ECTRN_BioBudget_production(1, :);
ECTRN_BioBudget_senescence      = ECTRN_BioBudget_production(3, :);
ECTRN_BioBudget_BA              = ECTRN_BioBudget_production(4, :);
ECTRN_BioBudget_EM              = ECTRN_BioBudget_production(5, :);

% step 8d: ECOTRAN addresses ----------------------------------------------
looky_NO3_ECTRN                     = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_NO3);
looky_plgcNH4_ECTRN                 = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_plgcNH4);
looky_bnthNH4_ECTRN                 = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_bnthNH4);
looky_ANYPrimaryProd_ECTRN          = find(floor(ECOTRAN.GroupType)     == ECOTRAN.GroupTypeDef_ANYPrimaryProd);
looky_eggs_ECTRN                    = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_eggs);
looky_fishery_ECTRN                 = find(floor(ECOTRAN.GroupType)     == ECOTRAN.GroupTypeDef_fishery);
looky_detritus_ECTRN                = find(floor(ECOTRAN.GroupType)     == ECOTRAN.GroupTypeDef_ANYDetritus);       % all ECOTRAN detritus, excluding eggs
looky_PelagicDetritus_ECTRN         = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_terminalPlgcDetr);  % terminal pelagic detritus group(s)
looky_BenthicDetritus_ECTRN         = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_terminalBnthDetr);	% terminal benthic detritus group(s)
looky_offal_ECTRN                   = find(ECOTRAN.GroupType            == ECOTRAN.GroupTypeDef_offal);             % offal group(s) in EwE
looky_OtherDetritus_ECTRN           = find((floor(ECOTRAN.GroupType)    == ECOTRAN.GroupTypeDef_ANYDetritus) & (ECOTRAN.GroupType ~= ECOTRAN.GroupTypeDef_terminalPlgcDetr) & (ECOTRAN.GroupType ~= ECOTRAN.GroupTypeDef_terminalBnthDetr)); % non-terminal, non-egg detritus groups
looky_DetritusAndEggs_ECTRN         = find(floor(ECOTRAN.GroupType)     == ECOTRAN.GroupTypeDef_ANYDetritus | floor(ECOTRAN.GroupType) == ECOTRAN.GroupTypeDef_eggs);           % will need this info later; (4-1-2014 includes eggs)
looky_DetritusInDetritus_ECTRN      = find((ismember(looky_DetritusAndEggs_ECTRN, (looky_DetritusAndEggs_ECTRN(~ismember(looky_DetritusAndEggs_ECTRN, looky_eggs_ECTRN))))));	% non-egg addresses in ECTRN_EnergyBudget_matrix and ECTRN_DetritusFate matrix
looky_EggsInDetritus                = find((ismember(looky_DetritusAndEggs_ECTRN, (looky_DetritusAndEggs_ECTRN(ismember(looky_DetritusAndEggs_ECTRN, looky_eggs_ECTRN))))));    % egg addresses in ECTRN_EnergyBudget_matrix and ECTRN_DetritusFate matrix
looky_AllNutrients_ECTRN            = [looky_NO3_ECTRN; looky_plgcNH4_ECTRN; looky_bnthNH4_ECTRN];                  % technically, NO3 is not metabolism, but there is no flow from any group to NO3 in EnergyBudget_matrix so this wil not cause a problem
looky_NonPredation_ECTRN            = [looky_AllNutrients_ECTRN; looky_eggs_ECTRN; looky_detritus_ECTRN];
looky_predation_ECTRN               = 1:num_ECTRNgroups;
looky_predation_ECTRN(looky_NonPredation_ECTRN) = [];

% step 8e: ECOTRAN fates --------------------------------------------------
ECTRN_DetritusFate_feces        = ECOTRAN.DetritusFate_feces;       % (2D matrix: 2 X num_ECTRNgroups); terminal pelagic & benthic detritus
ECTRN_DetritusFate_senescence   = ECOTRAN.DetritusFate_senescence;  % (2D matrix: 2 X num_ECTRNgroups); terminal pelagic & benthic detritus
ECTRN_ExcretionFat              = ECOTRAN.ExcretionFate;            % (2D matrix: 2 X num_ECTRNgroups); pelagic NH4 & benthic NH4
ECTRN_EggFate                   = ECOTRAN.EggFate;                  % (2D matrix: num_eggs X num_ECTRNgroups)

% step 8f: other ECOTRAN terms --------------------------------------------
% FFF: for the future
%       ECOTRAN.ProductionLossScaler
%       ECOTRAN.RetentionScaler
%       ECOTRAN.FunctionalResponseParams

% step 8g: final adjustments to CV values ---------------------------------
CV_senescence(looky_fishery_ECTRN)          = 0; % no fishery senescence
CV_eggs(looky_ANYPrimaryProd_ECTRN)         = 0; % no primary producer eggs
CV_eggs(looky_fishery_ECTRN)                = 0; % no fishery eggs
CV_eggs(looky_detritus_ECTRN)               = 0; % no detritus eggs
CV_eggs(looky_eggs_ECTRN)                   = 0; % no egg eggs




% *************************************************************************
% STEP 9: build-up CV_A matrix---------------------------------------------
CV_EnergyBudget_matrix                                  = CV_A;         % (2D matrix: num_ECTRNgroups X num_ECTRNgroups)

% step 9a: metabolism (NH4 pools) -----------------------------------------
CV_EnergyBudget_matrix(looky_plgcNH4_ECTRN, :)          = CV_MTBLSM;	% (2D matrix: num_ECTRNgroups X num_ECTRNgroups); NOTE: its OK to have CV for both NH4 pools even though only 1 pool exists for any specific group
CV_EnergyBudget_matrix(looky_bnthNH4_ECTRN, :)          = CV_MTBLSM;    % (2D matrix: num_ECTRNgroups X num_ECTRNgroups)
% QQQ distribute between NH4 pools based on fate


% step 9b: eggs -----------------------------------------------------------
CV_EnergyBudget_matrix(looky_eggs_ECTRN, :)             = repmat(CV_eggs, [num_eggs, 1]); % (2D matrix: num_ECTRNgroups X num_ECTRNgroups);

% step 9c: feces & senescence ---------------------------------------------
%           REMINDER: CV  = STD / mean
%               STD = CV * mean
%               VAR = STD^2
ECTRN_EnergyBudget_matrix_detritus                      = ECTRN_EnergyBudget_matrix(looky_detritus_ECTRN, :);
ECTRN_STD_feces                                         = ECTRN_EnergyBudget_matrix_detritus .* repmat(CV_feces, [num_detritus, 1]);
ECTRN_VAR_feces                                         = ECTRN_STD_feces.^2;
ECTRN_STD_senescence                                    = ECTRN_EnergyBudget_matrix_detritus .* repmat(CV_senescence, [num_detritus, 1]);
ECTRN_VAR_senescence                                    = ECTRN_STD_senescence.^2;
ECTRN_VAR_TotalDetritus                                 = ECTRN_VAR_feces + ECTRN_VAR_senescence; 
ECTRN_STD_TotalDetritus                                 = sqrt(ECTRN_VAR_TotalDetritus);
ECTRN_CV_TotalDetritus                                  = ECTRN_STD_TotalDetritus ./ ECTRN_EnergyBudget_matrix_detritus;
ECTRN_CV_TotalDetritus(isnan(ECTRN_CV_TotalDetritus))   = 0; % remove div/0 error NaNs
CV_EnergyBudget_matrix(looky_detritus_ECTRN, :)         = ECTRN_CV_TotalDetritus;

% repeat for terminal detritus groups in order to consider ECOTRAN detritus fates; as always in ECOTRAN code, this assumes terminal pelagic detritus in row 1 and terminal benthic detritus in row 2
ECTRN_EnergyBudget_matrix_detritus                      = ECTRN_EnergyBudget_matrix([looky_PelagicDetritus_ECTRN; looky_BenthicDetritus_ECTRN], :);
ECTRN_STD_feces                                         = (ECTRN_EnergyBudget_matrix_detritus .* ECTRN_DetritusFate_feces)      .* repmat(CV_feces, [2, 1]);
ECTRN_VAR_feces                                         = ECTRN_STD_feces.^2;
ECTRN_STD_senescence                                    = (ECTRN_EnergyBudget_matrix_detritus .* ECTRN_DetritusFate_senescence) .* repmat(CV_senescence, [2, 1]);
ECTRN_VAR_senescence                                    = ECTRN_STD_senescence.^2;
ECTRN_VAR_TotalDetritus                                 = ECTRN_VAR_feces + ECTRN_VAR_senescence; 
ECTRN_STD_TotalDetritus                                 = sqrt(ECTRN_VAR_TotalDetritus);
ECTRN_CV_TotalDetritus                                  = ECTRN_STD_TotalDetritus ./ ECTRN_EnergyBudget_matrix_detritus;
ECTRN_CV_TotalDetritus(isnan(ECTRN_CV_TotalDetritus))   = 0; % remove div/0 error NaNs
CV_EnergyBudget_matrix([looky_PelagicDetritus_ECTRN; looky_BenthicDetritus_ECTRN], :)         = ECTRN_CV_TotalDetritus;

% step 9d: implicit bacterial metabolism of terminal detritus groups ------
CV_EnergyBudget_matrix([looky_plgcNH4_ECTRN looky_bnthNH4_ECTRN], [looky_PelagicDetritus_ECTRN looky_BenthicDetritus_ECTRN]) = PEDIGREE_BacterialMTBLSM;
CV_MTBLSM([looky_PelagicDetritus_ECTRN looky_BenthicDetritus_ECTRN])                                                         = PEDIGREE_BacterialMTBLSM;

% step 9e: primary producer uptake of nutrients ---------------------------
CV_EnergyBudget_matrix(looky_ANYPrimaryProd_ECTRN, [looky_NO3_ECTRN looky_plgcNH4_ECTRN looky_bnthNH4_ECTRN]) = PEDIGREE_NutrientUptake;
CV_predation([looky_NO3_ECTRN looky_plgcNH4_ECTRN looky_bnthNH4_ECTRN])                                       = PEDIGREE_NutrientUptake;
% FFF in future, could add non-biological oxidation (reduction?) of NH4 to NO3

% step 9f: zero-out CV_A matrix cells that are 0 in EnergyBudget_matrix  --
zero_scaler                     = ECTRN_EnergyBudget_matrix;
zero_scaler(zero_scaler>0)      = 1;
CV_EnergyBudget_matrix          = CV_EnergyBudget_matrix .* zero_scaler;

% step 9g: filter out tiny error of ECOTRAN fisheries senescence ----------
ECTRN_BioBudget_senescence(1, looky_fishery_ECTRN) = 0; % (remove negative values in 17th decimal place)





% *************************************************************************
% STEP 10: build consumption budget----------------------------------------
%   row 1: feces
%   row 2: metabolism
%   row 3: eggs
%   row 4: predation
%   row 5: senescence
%   row 6: BA
%   row 7: EM
CV_ConsumptionBudget                = zeros(7, num_ECTRNgroups) * NaN;  % initialize
CV_ConsumptionBudget(1, :)          = CV_feces;                         % feces
CV_ConsumptionBudget(2, :)          = CV_MTBLSM;                        % metabolism
CV_ConsumptionBudget(3, :)          = CV_eggs;                          % eggs
CV_ConsumptionBudget(4, :)          = CV_predation;                     % predation
CV_ConsumptionBudget(5, :)          = CV_senescence;                    % senescence
CV_ConsumptionBudget(6, :)          = CV_BA;                            % BA
CV_ConsumptionBudget(7, :)          = CV_EM;                            % EM
% QQQ EM needs to consider import diet info (can ignore for now since in all models they are small)

ECTRN_ConsumptionBudget         	= zeros(7, num_ECTRNgroups) * NaN;	% initialize
ECTRN_ConsumptionBudget(1, :)     	= ECTRN_BioBudget_feces;          	% feces
ECTRN_ConsumptionBudget(2, :)      	= ECTRN_BioBudget_MTBLSM;           % metabolism
ECTRN_ConsumptionBudget(3, :)    	= ECTRN_BioBudget_eggs;             % eggs
ECTRN_ConsumptionBudget(4, :)     	= ECTRN_BioBudget_predation;        % predation
ECTRN_ConsumptionBudget(5, :)      	= ECTRN_BioBudget_senescence;       % senescence
ECTRN_ConsumptionBudget(6, :)      	= ECTRN_BioBudget_BA;               % BA
ECTRN_ConsumptionBudget(7, :)     	= ECTRN_BioBudget_EM;               % EM

% step 9f: zero-out CV_A matrix cells that are 0 in EnergyBudget_matrix  --
zero_scaler                         = ECTRN_ConsumptionBudget;
zero_scaler(zero_scaler>0)          = 1;
CV_ConsumptionBudget                = CV_ConsumptionBudget .* zero_scaler;





% *************************************************************************
% STEP 11: package for export----------------------------------------------
ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix         = CV_EnergyBudget_matrix;
ECOTRAN_PEDIGREE.CV_ConsumptionBudget           = CV_ConsumptionBudget;
ECOTRAN_PEDIGREE.ECTRN_ConsumptionBudget        = ECTRN_ConsumptionBudget;

ECOTRAN_PEDIGREE.looky_AllNutrients             = looky_AllNutrients_ECTRN;     % technically, NO3 is not metabolism, but there is no flow from any group to NO3 in EnergyBudget_matrix so this wil not cause a problem
ECOTRAN_PEDIGREE.looky_detritus                 = looky_detritus_ECTRN;
ECOTRAN_PEDIGREE.looky_eggs                     = looky_eggs_ECTRN;
ECOTRAN_PEDIGREE.looky_predation                = looky_predation_ECTRN;
ECOTRAN_PEDIGREE.looky_fishery                  = looky_fishery_ECTRN;
ECOTRAN_PEDIGREE.looky_PelagicDetritus          = looky_PelagicDetritus_ECTRN;
ECOTRAN_PEDIGREE.looky_BenthicDetritus          = looky_BenthicDetritus_ECTRN;

ECOTRAN_PEDIGREE.PEDIGREE_other                 = PEDIGREE_other; % "other" defined input pedigree terms for the whole model in general, pack these terms to pass along to other functions


% end m-file***************************************************************