function [SurfaceRadiation] = f_LightIntensity(LightParams)
% calculate light intensity
% Solar radiation, based on Brock (1981, Ecol. Modelling 14:1-19)
% takes:
%       t_grid
%       latitude
%       SolarConstant  Solar constant (W/m2) recent average = 1366 (Froehlich & Lean (1998, Geographical Research Letters 25(23): 4377-4380))
%       par_frac
%       surf_trans
%       cloud_trans
% returns:
%       SurfaceRadiation.sunrise                 time of sunrise (time in h from midnight; 12 = noon, set as a default)
%       SurfaceRadiation.sunset                  time of sunset  (time in h from midnight; 12 = noon, set as a default)
%       SurfaceRadiation.day_length              hours of daylight
%       SurfaceRadiation.current_light           surface solar raditation at current time (W/m2)
%       SurfaceRadiation.daily_integrated_light  daily integrated solar raditation at ocean surface (W m^-2 d^-1)
%       SurfaceRadiation.daily_average_light     daily integrated solar raditation at ocean surface averaged across 24 hours (W m^-2 h^-1)
% revision date: 3-5-2014

% trigonmetric functions are set for degrees
julian_day = floor(LightParams.t_grid); % not necessary to correct t for multiple years
D1         = 23.45 * sind(360 * (284 + julian_day) / 365); % declination, angular distance between sun and equator at solar noon (degrees); eq. 1
W1         = acosd(-(tand(D1) * tand(LightParams.latitude)));  % sunset hour-angle, angle between setting sun and the south point (degrees); eq. 3
day_length = (W1 / 15) * 2;  % day length (h); eq. 4
sunrise    = 12 - 1/2 * day_length; % time of sunrise
sunset     = 12 + 1/2 * day_length;
R1         = 1 ./ (1 + (0.033 * cosd(360 * julian_day / 365))).^0.5; % radius vector, used to correct for ellipticity of Earth's orbit (dimensionless); eq. 2; 

% current light intensity at time, local_time
local_time = round((LightParams.t_grid - julian_day) * 24); % local_time is time in h from midnight (12 = noon, set as a default)
W2         = (local_time - 12) * 15; % hour-angle (degrees); eq. 5
I1         = (LightParams.SolarConstant ./ R1.^2) .* (sind(D1) .* sind(LightParams.latitude) + cosd(D1) .* cosd(LightParams.latitude) .* cosd(W2)); % top-of-atmosphere solar radiation at local_time (W/m2); eq. 7
I1(I1 < 0) = 0; % filter out negative values (caused by after sunset times)

% daily integrated light intensity at time, julian_day
I6         = (24 / pi) * (LightParams.SolarConstant ./ R1.^2) .* (W1 .* (pi / 180) .* sind(LightParams.latitude) .* sind(D1) + sind(W1) .* cosd(LightParams.latitude) .* cosd(D1));  % integrated daily top-of-atmosphere (W m^-2 d^-1)
I6_average = (I6 / 24) * LightParams.par_frac * LightParams.surf_trans * LightParams.cloud_trans; % daily integrated solar raditation at ocean surface averaged across 24 hours (W m^-2 h^-1)

current_light          = I1 * (LightParams.par_frac * LightParams.surf_trans * LightParams.cloud_trans); % surface solar raditation at current time (W/m2)
daily_integrated_light = I6 * (LightParams.par_frac * LightParams.surf_trans * LightParams.cloud_trans); % daily integrated solar raditation at ocean surface (W m^-2 d^-1)
daily_average_light    = I6_average; % daily integrated solar raditation at ocean surface averaged across 24 hours (W m^-2 h^-1)

% pack results for export
SurfaceRadiation.sunrise                = sunrise;            % time of sunrise (time in h from midnight; 12 = noon, set as a default)
SurfaceRadiation.sunset                 = sunset;             % time of sunset  (time in h from midnight; 12 = noon, set as a default)
SurfaceRadiation.day_length             = day_length;     % hours of daylight
SurfaceRadiation.current_light          = current_light; % surface solar raditation at current time (W/m2)
SurfaceRadiation.daily_integrated_light = daily_integrated_light; % daily integrated solar raditation at ocean surface (W m^-2 d^-1)
SurfaceRadiation.daily_average_light    = daily_average_light; % daily integrated solar raditation at ocean surface averaged across 24 hours (W m^-2 h^-1)

% end m-file***************************************************************