function [EnergyBudget_scenario Production_scenario BuildGroup max_ScaleFactor] = f_ScenarioGenerator_08302013(ScenarioConditions, ECOTRAN)
% by Jim Ruzicka
% generate a modified scenario matrix (or set of matrices)
% calculate productivity vectors for scenario and ratios compared to un-modified matrices
% calls:
%   f_WebProductivityWLoss
% takes:
%   ScenarioConditions
%           ScenarioConditions.trgt_producer            (column(s) of producer groups to modify
%           ScenarioConditions.modify_consumer          (row(s) of consumer group to modify
%           ScenarioConditions.offset_consumer          (row(s) of consumer group to modify to offset
%                                                           change in modify_consumer grp(s); can be [] if offset is to be distributed among ALL consumer groups)
%           ScenarioConditions.ScaleFactor              (factor by which to re-scale production to modify_consumer group(s)
%           ScenarioConditions.EnergyBudget_base        (EnergyBudget_base is a MonteCarlo set of EnergyBudget_matrix models 
%                                                               production, predation, or consumption matrix) (1 matrix or several stacked matrices as generated by Monte Carlo sampling)
%           ScenarioConditions.TransferEfficiency       (horizontal vector)
%           ScenarioConditions.InputProductionVector    (vertical vector)
%           ScenarioConditions.PhysicalLossFraction     (horizontal vector)
%           ScenarioConditions.force_offset             ('n' or 'y': 'y' means to force offset consumer fractions evenly, even if original value is 0,
%                                                           'n' means to distribute proportionally to original values
%   ECOTRAN
%           ECOTRAN.BioenergeticBudget_FecesDetail      (fate of feces within EnergyBudget_matrix)
%           ECOTRAN.GroupType
%           ECOTRAN.GroupType_detritus
%           ECOTRAN.GroupTypeDef_                       GroupType number codes (several of them)
% returns:
%           EnergyBudget_scenario            (set of modified production matrices)
%           Production_scenario         (production vector of each modified matrix) (matrix of number of groups X number of Arrays modified)
%           BuildGroup             (matrix of trophic group number X number of Arrays modified; needed for box plots)
%           max_ScaleFactor         (maximum allowed ScaleFactor for all producer groups; NaN where producer is not eaten by any modify_consumer (matrix of (1:num_trials, 1:E2E_clms)
% revision date: 8-30-2013





% *************************************************************************
% STEP 1: operating parameters---------------------------------------------
minimum_residual = 0.001; % minimum residual scaling factor for offset consumers (largest factor by which an offset consumer's take can be scaled down)





% *************************************************************************
% STEP 2: unpack variables (preserve values within original structures)----
trgt_producer                  = ScenarioConditions.trgt_producer;
modify_consumer                = ScenarioConditions.modify_consumer;
offset_consumer                = ScenarioConditions.offset_consumer;
ScaleFactor                    = ScenarioConditions.ScaleFactor;
EnergyBudget_matrix            = ScenarioConditions.EnergyBudget_base;
TransferEfficiency             = ScenarioConditions.TransferEfficiency;
InputProductionVector          = ScenarioConditions.InputProductionVector;
PhysicalLossFraction           = ScenarioConditions.PhysicalLossFraction;
force_offset                   = ScenarioConditions.force_offset;
GroupType                      = ECOTRAN.GroupType;
GroupType_detritus             = ECOTRAN.GroupType_detritus;
BioenergeticBudget_FecesDetail = ECOTRAN.BioenergeticBudget_FecesDetail; % shows fate of feces within EnergyBudget_matrix (allows separation of feces from "OtherMortality" detritus)

if ScaleFactor < 0; error('negative scalers make no sense'); end





% *************************************************************************
% STEP 3: prep variables---------------------------------------------------
[E2E_rows, E2E_clms, num_trials] = size(EnergyBudget_matrix);
EnergyBudget_scenario            = zeros(E2E_rows, E2E_clms, num_trials); % initialize
max_ScaleFactor                  = zeros(num_trials, E2E_clms) * NaN; % initialize

% non-offset consumer list (the groups that DO NOT change)
non_offset_consumer                                            = 1:E2E_rows; % (vector)
non_offset_consumer([offset_consumer(:); modify_consumer(:)] ) = []; % remove offset_consumer and modify_consumer from non_offset_consumer list

% remove modify_consumer from offset_consumer list
looky_modify                  = find(ismember(offset_consumer, modify_consumer));
offset_consumer(looky_modify) = [];

% check if do TransferEfficiency, detritus_OtherMortality, and detritus_Feces have layers
[rows_TransferEfficiency, clms_TransferEfficiency] = size(TransferEfficiency);





% *************************************************************************
% STEP 4: find detritus, nutrient, & fishery groups------------------------
looky_TerminalPelagicDetritus    = find(GroupType          == ECOTRAN.GroupTypeDef_terminalPlgcDetr); % address in EnergyBudget_matrix
looky_TerminalBenthicDetritus    = find(GroupType          == ECOTRAN.GroupTypeDef_terminalBnthDetr); % address in EnergyBudget_matrix
looky_TerminalPelagic_DetrDetail = find(GroupType_detritus == ECOTRAN.GroupTypeDef_terminalPlgcDetr); % address in detritus detail
looky_TerminalBenthic_DetrDetail = find(GroupType_detritus == ECOTRAN.GroupTypeDef_terminalBnthDetr); % address in detritus detail
looky_nutrient                   = find(floor(GroupType)   == ECOTRAN.GroupTypeDef_ANYNitroNutr); % address in EnergyBudget_matrix
looky_fishery                    = find(floor(GroupType)   == ECOTRAN.GroupTypeDef_fishery); % address in EnergyBudget_matrix





% *************************************************************************
% STEP 5: process each Monte Carlo (or "type" model)-----------------------
for MonteCarlo_loop = 1:num_trials
    
    % step 5a: get current EnergyBudget_matrix & current_TransferEfficiency
    if rows_TransferEfficiency == 1
        target_layer = 1;
    else
        target_layer = MonteCarlo_loop;
    end
    
    current_TransferEfficiency  = TransferEfficiency(target_layer, :);
    current_FecesDetail         = BioenergeticBudget_FecesDetail(:, :, target_layer);
    current_matrix              = EnergyBudget_matrix(:, :, MonteCarlo_loop);
    original_matrix             = current_matrix;
    
    
    
    % step 5b: remove feces from current_matrix (EnergyBudget_matrix) because assimilation efficiency (feces fraction) does not change in scenarios
    % get feces contribution to terminal pelagic & terminal benthic detritus
    feces_pelagic = current_FecesDetail(looky_TerminalPelagic_DetrDetail, :); % feces contribution to terminal pelagic detritus
    feces_benthic = current_FecesDetail(looky_TerminalBenthic_DetrDetail, :); % feces contribution to terminal benthic detritus

    % pull feces out of terminal pelagic & terminal benthic detritus in modify_matrix
    current_matrix(looky_TerminalPelagicDetritus, :) = current_matrix(looky_TerminalPelagicDetritus, :) - feces_pelagic;
    current_matrix(looky_TerminalBenthicDetritus, :) = current_matrix(looky_TerminalBenthicDetritus, :) - feces_benthic;   
    
    % step 5c: estimate maximum allowed scaling factor upon each producer group
    offset_consumer_matrix                      = current_matrix(offset_consumer, 1:E2E_clms); % matrix (or vector) of offset consumer take
    min_offset_consumer_matrix                  = offset_consumer_matrix .* minimum_residual; % apply minimum trace on each element
    offset_consumer_freedom                     = offset_consumer_matrix - min_offset_consumer_matrix; % maximum amount that can be taken from offset consumers
    sum_offset_consumer_freedom                 = sum(offset_consumer_freedom, 1);
    modify_consumer_matrix                      = current_matrix(modify_consumer, 1:E2E_clms); % the original, un-scaled modify consumer take (matrix, vector, or scaler)
    sum_modify_consumer_matrix                  = sum(modify_consumer_matrix, 1);
    
    % QQQ need to think on this a bit more
    max_scaler                                  = 1 + (sum_offset_consumer_freedom ./ sum_modify_consumer_matrix); % how many more times can the modify consumers be scaled up
    max_scaler(sum_modify_consumer_matrix == 0) = NaN;
    if nanmin(max_scaler(trgt_producer)) < ScaleFactor; display('Warning: MAXIMUM SCALER EXCEEDED FOR AT LEAST ONE TARGET PRODUCER'); end
    max_scaler(isnan(max_scaler))               = ScaleFactor; % was 1 (8-23-2011)
    max_ScaleFactor(MonteCarlo_loop, 1:E2E_clms)     = max_scaler; % build up a record
    UseScaleFactor_vector(1, 1:E2E_clms)        = ScaleFactor;
    
    % QQQ need to think on this a bit more
    if ScaleFactor > 1
        looky_ExceededLimits = find(max_scaler < ScaleFactor);
        UseScaleFactor_vector(looky_ExceededLimits) = max_scaler(looky_ExceededLimits); % replace values in UseScaleFactor_vector where ScaleFactor exceedes max_scaler with max_scaler
    end
    
    UseScaleFactor_vector = UseScaleFactor_vector(trgt_producer); % trim UseScaleFactor_vector to just the target producer columns

    % step 5d: apply scaling factor
    modify_matrix = current_matrix;
    for scaling_loop = 1:length(modify_consumer)
        current_modify_consumer = modify_consumer(scaling_loop);
        modify_matrix(current_modify_consumer, trgt_producer) = modify_matrix(current_modify_consumer, trgt_producer) .* UseScaleFactor_vector;
    end
    
    delta_matrix = current_matrix - modify_matrix;
    sum_delta    = sum(delta_matrix, 1);
    
    % step 5e: change offset group(s); distribute based proportionally to original values
    if ~isempty(offset_consumer)
        offset_vectors = current_matrix(offset_consumer, :);
        
        if strcmp(force_offset, 'y')
            for offset_loop = 1:length(offset_consumer);
                offset_vectors(offset_loop, :) = offset_vectors(offset_loop, :) + (sum_delta ./ length(offset_consumer));
            end
        else
            sum_offset = sum(offset_vectors, 1);
            for offset_loop = 1:length(offset_consumer);
                offset_fraction = offset_vectors(offset_loop, :) ./ sum_offset;
                offset_fraction(isnan(offset_fraction)) = 0; % remove NaN caused by div/0
                offset_vectors(offset_loop, :) = offset_vectors(offset_loop, :) + (offset_fraction .* sum_delta);
         	  end
        end
    
        modify_matrix(offset_consumer, :) = offset_vectors;

    end % (~isempty(offset_consumer))
    
    
    % step 5f: paste feces back into the detritus rows of modify_matrix
    modify_matrix(looky_TerminalPelagicDetritus, :) = modify_matrix(looky_TerminalPelagicDetritus, :) + feces_pelagic; % add back pelagic feces
    modify_matrix(looky_TerminalBenthicDetritus, :) = modify_matrix(looky_TerminalBenthicDetritus, :) + feces_benthic; % add back benthic feces
    
    % step 5g: renormalize each column to original, base-model column sum (eliminates modifications > 1; redistributes offset among all consumers if offset groups are not defined)
    sum_modify_matrix   = sum(modify_matrix, 1);
    sum_original_matrix = sum(original_matrix, 1);
    for row_loop = 1:E2E_rows
        modify_matrix(row_loop, :) = (modify_matrix(row_loop, :) ./ sum_modify_matrix) .* sum_original_matrix; % the term sum(original_matrix, 1) allows for producers in base matrix that do not sum to 1 (i.e., fisheries)
    end
    
    modify_matrix(isnan(modify_matrix))         = 0; % replace NaN with 0
    EnergyBudget_scenario(:, :, MonteCarlo_loop) = modify_matrix;
    
    % step 5h: calculate production rates using modified, scenario matrix
    Production_scenario(:, MonteCarlo_loop) = f_WebProductivityWLoss(current_TransferEfficiency, modify_matrix, InputProductionVector, PhysicalLossFraction);
    BuildGroup(1:E2E_rows, MonteCarlo_loop) = (1:E2E_rows)';
    
end % (MonteCarlo_loop)





% end m-file***************************************************************